@@ -6,7 +6,7 @@ mod categorical;
 use std::ops::{BitAnd, Not};
 
 use arrow::array::BooleanArray;
-use arrow::bitmap::MutableBitmap;
+use arrow::bitmap::{Bitmap, MutableBitmap};
 use arrow::compute;
 use num_traits::{NumCast, ToPrimitive};
 use polars_compute::comparisons::{TotalEqKernel, TotalOrdKernel};
@@ -16,7 +16,7 @@ use crate::series::implementations::null::NullChunked;
 use crate::series::IsSorted;
 use crate::utils::align_chunks_binary;
 
-impl<T> ChunkCompare<&ChunkedArray<T>> for ChunkedArray<T>
+impl<T> ChunkCompareEq<&ChunkedArray<T>> for ChunkedArray<T>
 where
     T: PolarsNumericType,
     T::Array: TotalOrdKernel<Scalar = T::Native> + TotalEqKernel<Scalar = T::Native>,
@@ -126,6 +126,14 @@ where
             ),
         }
     }
+}
+
+impl<T> ChunkCompareIneq<&ChunkedArray<T>> for ChunkedArray<T>
+where
+    T: PolarsNumericType,
+    T::Array: TotalOrdKernel<Scalar = T::Native> + TotalEqKernel<Scalar = T::Native>,
+{
+    type Item = BooleanChunked;
 
     fn lt(&self, rhs: &ChunkedArray<T>) -> BooleanChunked {
         // Broadcast.
@@ -188,7 +196,7 @@ where
     }
 }
 
-impl ChunkCompare<&NullChunked> for NullChunked {
+impl ChunkCompareEq<&NullChunked> for NullChunked {
     type Item = BooleanChunked;
 
     fn equal(&self, rhs: &NullChunked) -> Self::Item {
@@ -206,6 +214,10 @@ impl ChunkCompare<&NullChunked> for NullChunked {
     fn not_equal_missing(&self, rhs: &NullChunked) -> Self::Item {
         BooleanChunked::full(self.name().clone(), false, get_broadcast_length(self, rhs))
     }
+}
+
+impl ChunkCompareIneq<&NullChunked> for NullChunked {
+    type Item = BooleanChunked;
 
     fn gt(&self, rhs: &NullChunked) -> Self::Item {
         BooleanChunked::full_null(self.name().clone(), get_broadcast_length(self, rhs))
@@ -234,7 +246,7 @@ fn get_broadcast_length(lhs: &NullChunked, rhs: &NullChunked) -> usize {
     }
 }
 
-impl ChunkCompare<&BooleanChunked> for BooleanChunked {
+impl ChunkCompareEq<&BooleanChunked> for BooleanChunked {
     type Item = BooleanChunked;
 
     fn equal(&self, rhs: &BooleanChunked) -> BooleanChunked {
@@ -348,6 +360,10 @@ impl ChunkCompare<&BooleanChunked> for BooleanChunked {
             ),
         }
     }
+}
+
+impl ChunkCompareIneq<&BooleanChunked> for BooleanChunked {
+    type Item = BooleanChunked;
 
     fn lt(&self, rhs: &BooleanChunked) -> BooleanChunked {
         // Broadcast.
@@ -410,7 +426,7 @@ impl ChunkCompare<&BooleanChunked> for BooleanChunked {
     }
 }
 
-impl ChunkCompare<&StringChunked> for StringChunked {
+impl ChunkCompareEq<&StringChunked> for StringChunked {
     type Item = BooleanChunked;
 
     fn equal(&self, rhs: &StringChunked) -> BooleanChunked {
@@ -424,9 +440,14 @@ impl ChunkCompare<&StringChunked> for StringChunked {
     fn not_equal(&self, rhs: &StringChunked) -> BooleanChunked {
         self.as_binary().not_equal(&rhs.as_binary())
     }
+
     fn not_equal_missing(&self, rhs: &StringChunked) -> BooleanChunked {
         self.as_binary().not_equal_missing(&rhs.as_binary())
     }
+}
+
+impl ChunkCompareIneq<&StringChunked> for StringChunked {
+    type Item = BooleanChunked;
 
     fn gt(&self, rhs: &StringChunked) -> BooleanChunked {
         self.as_binary().gt(&rhs.as_binary())
@@ -445,7 +466,7 @@ impl ChunkCompare<&StringChunked> for StringChunked {
     }
 }
 
-impl ChunkCompare<&BinaryChunked> for BinaryChunked {
+impl ChunkCompareEq<&BinaryChunked> for BinaryChunked {
     type Item = BooleanChunked;
 
     fn equal(&self, rhs: &BinaryChunked) -> BooleanChunked {
@@ -551,6 +572,10 @@ impl ChunkCompare<&BinaryChunked> for BinaryChunked {
             ),
         }
     }
+}
+
+impl ChunkCompareIneq<&BinaryChunked> for BinaryChunked {
+    type Item = BooleanChunked;
 
     fn lt(&self, rhs: &BinaryChunked) -> BooleanChunked {
         // Broadcast.
@@ -613,93 +638,131 @@ impl ChunkCompare<&BinaryChunked> for BinaryChunked {
     }
 }
 
-#[doc(hidden)]
-fn _list_comparison_helper<F>(lhs: &ListChunked, rhs: &ListChunked, op: F) -> BooleanChunked
+fn _list_comparison_helper<F, B>(
+    lhs: &ListChunked,
+    rhs: &ListChunked,
+    op: F,
+    broadcast_op: B,
+    missing: bool,
+    is_ne: bool,
+) -> BooleanChunked
 where
-    F: Fn(Option<&Series>, Option<&Series>) -> Option<bool>,
+    F: Fn(&ListArray<i64>, &ListArray<i64>) -> Bitmap,
+    B: Fn(&ListArray<i64>, &Box<dyn Array>) -> Bitmap,
 {
     match (lhs.len(), rhs.len()) {
         (_, 1) => {
-            let right = rhs.get_as_series(0).map(|s| s.with_name(PlSmallStr::EMPTY));
-            lhs.amortized_iter()
-                .map(|left| op(left.as_ref().map(|us| us.as_ref()), right.as_ref()))
-                .collect_trusted()
+            let right = rhs.chunks()[0]
+                .as_any()
+                .downcast_ref::<ListArray<i64>>()
+                .unwrap();
+
+            if !right.validity().map_or(true, |v| v.get(0).unwrap()) {
+                if missing {
+                    if is_ne {
+                        return lhs.is_not_null();
+                    } else {
+                        return lhs.is_null();
+                    }
+                } else {
+                    return BooleanChunked::full_null(PlSmallStr::EMPTY, lhs.len());
+                }
+            }
+
+            let values = right.values().sliced(
+                (*right.offsets().first()).try_into().unwrap(),
+                right.offsets().range().try_into().unwrap(),
+            );
+
+            if missing {
+                arity::unary_mut_with_options(lhs, |a| broadcast_op(a, &values).into())
+            } else {
+                arity::unary_mut_values(lhs, |a| broadcast_op(a, &values).into())
+            }
         },
         (1, _) => {
-            let left = lhs.get_as_series(0).map(|s| s.with_name(PlSmallStr::EMPTY));
-            rhs.amortized_iter()
-                .map(|right| op(left.as_ref(), right.as_ref().map(|us| us.as_ref())))
-                .collect_trusted()
+            let left = lhs.chunks()[0]
+                .as_any()
+                .downcast_ref::<ListArray<i64>>()
+                .unwrap();
+
+            if !left.validity().map_or(true, |v| v.get(0).unwrap()) {
+                if missing {
+                    if is_ne {
+                        return rhs.is_not_null();
+                    } else {
+                        return rhs.is_null();
+                    }
+                } else {
+                    return BooleanChunked::full_null(PlSmallStr::EMPTY, rhs.len());
+                }
+            }
+
+            let values = left.values().sliced(
+                (*left.offsets().first()).try_into().unwrap(),
+                left.offsets().range().try_into().unwrap(),
+            );
+
+            if missing {
+                arity::unary_mut_with_options(rhs, |a| broadcast_op(a, &values).into())
+            } else {
+                arity::unary_mut_values(rhs, |a| broadcast_op(a, &values).into())
+            }
+        },
+        _ => {
+            if missing {
+                arity::binary_mut_with_options(lhs, rhs, |a, b| op(a, b).into(), PlSmallStr::EMPTY)
+            } else {
+                arity::binary_mut_values(lhs, rhs, |a, b| op(a, b).into(), PlSmallStr::EMPTY)
+            }
         },
-        _ => lhs
-            .amortized_iter()
-            .zip(rhs.amortized_iter())
-            .map(|(left, right)| {
-                op(
-                    left.as_ref().map(|us| us.as_ref()),
-                    right.as_ref().map(|us| us.as_ref()),
-                )
-            })
-            .collect_trusted(),
     }
 }
 
-impl ChunkCompare<&ListChunked> for ListChunked {
+impl ChunkCompareEq<&ListChunked> for ListChunked {
     type Item = BooleanChunked;
     fn equal(&self, rhs: &ListChunked) -> BooleanChunked {
-        let _series_equals = |lhs: Option<&Series>, rhs: Option<&Series>| match (lhs, rhs) {
-            (Some(l), Some(r)) => Some(l.equals(r)),
-            _ => None,
-        };
-
-        _list_comparison_helper(self, rhs, _series_equals)
+        _list_comparison_helper(
+            self,
+            rhs,
+            TotalEqKernel::tot_eq_kernel,
+            TotalEqKernel::tot_eq_kernel_broadcast,
+            false,
+            false,
+        )
     }
 
     fn equal_missing(&self, rhs: &ListChunked) -> BooleanChunked {
-        let _series_equals_missing = |lhs: Option<&Series>, rhs: Option<&Series>| match (lhs, rhs) {
-            (Some(l), Some(r)) => Some(l.equals_missing(r)),
-            (None, None) => Some(true),
-            _ => Some(false),
-        };
-
-        _list_comparison_helper(self, rhs, _series_equals_missing)
+        _list_comparison_helper(
+            self,
+            rhs,
+            TotalEqKernel::tot_eq_missing_kernel,
+            TotalEqKernel::tot_eq_missing_kernel_broadcast,
+            true,
+            false,
+        )
     }
 
     fn not_equal(&self, rhs: &ListChunked) -> BooleanChunked {
-        let _series_not_equal = |lhs: Option<&Series>, rhs: Option<&Series>| match (lhs, rhs) {
-            (Some(l), Some(r)) => Some(!l.equals(r)),
-            _ => None,
-        };
-
-        _list_comparison_helper(self, rhs, _series_not_equal)
+        _list_comparison_helper(
+            self,
+            rhs,
+            TotalEqKernel::tot_ne_kernel,
+            TotalEqKernel::tot_ne_kernel_broadcast,
+            false,
+            true,
+        )
     }
 
     fn not_equal_missing(&self, rhs: &ListChunked) -> BooleanChunked {
-        let _series_not_equal_missing =
-            |lhs: Option<&Series>, rhs: Option<&Series>| match (lhs, rhs) {
-                (Some(l), Some(r)) => Some(!l.equals_missing(r)),
-                (None, None) => Some(false),
-                _ => Some(true),
-            };
-
-        _list_comparison_helper(self, rhs, _series_not_equal_missing)
-    }
-
-    // The following are not implemented because gt, lt comparison of series don't make sense.
-    fn gt(&self, _rhs: &ListChunked) -> BooleanChunked {
-        unimplemented!()
-    }
-
-    fn gt_eq(&self, _rhs: &ListChunked) -> BooleanChunked {
-        unimplemented!()
-    }
-
-    fn lt(&self, _rhs: &ListChunked) -> BooleanChunked {
-        unimplemented!()
-    }
-
-    fn lt_eq(&self, _rhs: &ListChunked) -> BooleanChunked {
-        unimplemented!()
+        _list_comparison_helper(
+            self,
+            rhs,
+            TotalEqKernel::tot_ne_missing_kernel,
+            TotalEqKernel::tot_ne_missing_kernel_broadcast,
+            true,
+            true,
+        )
     }
 }
 
@@ -709,25 +772,56 @@ fn struct_helper<F, R>(
     b: &StructChunked,
     op: F,
     reduce: R,
-    value: bool,
+    op_is_ne: bool,
+    is_missing: bool,
 ) -> BooleanChunked
 where
     F: Fn(&Series, &Series) -> BooleanChunked,
     R: Fn(BooleanChunked, BooleanChunked) -> BooleanChunked,
 {
-    if a.len() != b.len() || a.struct_fields().len() != b.struct_fields().len() {
-        // polars_ensure!(a.len() == 1 || b.len() == 1, ShapeMismatch: "length lhs: {}, length rhs: {}", a.len(), b.len());
-        BooleanChunked::full(PlSmallStr::EMPTY, value, a.len())
+    let len_a = a.len();
+    let len_b = b.len();
+    let broadcasts = len_a == 1 || len_b == 1;
+    if (a.len() != b.len() && !broadcasts) || a.struct_fields().len() != b.struct_fields().len() {
+        BooleanChunked::full(PlSmallStr::EMPTY, op_is_ne, a.len())
     } else {
         let (a, b) = align_chunks_binary(a, b);
+
         let mut out = a
             .fields_as_series()
             .iter()
             .zip(b.fields_as_series().iter())
             .map(|(l, r)| op(l, r))
-            .reduce(reduce)
-            .unwrap();
-        if a.null_count() > 0 || b.null_count() > 0 {
+            .reduce(&reduce)
+            .unwrap_or_else(|| BooleanChunked::full(PlSmallStr::EMPTY, !op_is_ne, a.len()));
+
+        if is_missing && (a.has_nulls() || b.has_nulls()) {
+            // Do some allocations so that we can use the Series dispatch, it otherwise
+            // gets complicated dealing with combinations of ==, != and broadcasting.
+            let default = || {
+                BooleanChunked::with_chunk(PlSmallStr::EMPTY, BooleanArray::from_slice([true]))
+                    .into_series()
+            };
+            let validity_to_series = |x| unsafe {
+                BooleanChunked::with_chunk(
+                    PlSmallStr::EMPTY,
+                    BooleanArray::from_inner_unchecked(ArrowDataType::Boolean, x, None),
+                )
+                .into_series()
+            };
+
+            out = reduce(
+                out,
+                op(
+                    &a.rechunk_validity()
+                        .map_or_else(default, validity_to_series),
+                    &b.rechunk_validity()
+                        .map_or_else(default, validity_to_series),
+                ),
+            )
+        }
+
+        if !is_missing && (a.null_count() > 0 || b.null_count() > 0) {
             let mut a = a.into_owned();
             a.zip_outer_validity(&b);
             unsafe {
@@ -736,20 +830,22 @@ where
                 }
             }
         }
+
         out
     }
 }
 
 #[cfg(feature = "dtype-struct")]
-impl ChunkCompare<&StructChunked> for StructChunked {
+impl ChunkCompareEq<&StructChunked> for StructChunked {
     type Item = BooleanChunked;
     fn equal(&self, rhs: &StructChunked) -> BooleanChunked {
         struct_helper(
             self,
             rhs,
-            |l, r| l.equal(r).unwrap(),
+            |l, r| l.equal_missing(r).unwrap(),
             |a, b| a.bitand(b),
             false,
+            false,
         )
     }
 
@@ -760,6 +856,7 @@ impl ChunkCompare<&StructChunked> for StructChunked {
             |l, r| l.equal_missing(r).unwrap(),
             |a, b| a.bitand(b),
             false,
+            true,
         )
     }
 
@@ -767,9 +864,10 @@ impl ChunkCompare<&StructChunked> for StructChunked {
         struct_helper(
             self,
             rhs,
-            |l, r| l.not_equal(r).unwrap(),
+            |l, r| l.not_equal_missing(r).unwrap(),
             |a, b| a | b,
             true,
+            false,
         )
     }
 
@@ -780,77 +878,129 @@ impl ChunkCompare<&StructChunked> for StructChunked {
             |l, r| l.not_equal_missing(r).unwrap(),
             |a, b| a | b,
             true,
+            true,
         )
     }
 }
 
 #[cfg(feature = "dtype-array")]
-impl ChunkCompare<&ArrayChunked> for ArrayChunked {
+fn _array_comparison_helper<F, B>(
+    lhs: &ArrayChunked,
+    rhs: &ArrayChunked,
+    op: F,
+    broadcast_op: B,
+    missing: bool,
+    is_ne: bool,
+) -> BooleanChunked
+where
+    F: Fn(&FixedSizeListArray, &FixedSizeListArray) -> Bitmap,
+    B: Fn(&FixedSizeListArray, &Box<dyn Array>) -> Bitmap,
+{
+    match (lhs.len(), rhs.len()) {
+        (_, 1) => {
+            let right = rhs.chunks()[0]
+                .as_any()
+                .downcast_ref::<FixedSizeListArray>()
+                .unwrap();
+
+            if !right.validity().map_or(true, |v| v.get(0).unwrap()) {
+                if missing {
+                    if is_ne {
+                        return lhs.is_not_null();
+                    } else {
+                        return lhs.is_null();
+                    }
+                } else {
+                    return BooleanChunked::full_null(PlSmallStr::EMPTY, lhs.len());
+                }
+            }
+
+            if missing {
+                arity::unary_mut_with_options(lhs, |a| broadcast_op(a, right.values()).into())
+            } else {
+                arity::unary_mut_values(lhs, |a| broadcast_op(a, right.values()).into())
+            }
+        },
+        (1, _) => {
+            let left = lhs.chunks()[0]
+                .as_any()
+                .downcast_ref::<FixedSizeListArray>()
+                .unwrap();
+
+            if !left.validity().map_or(true, |v| v.get(0).unwrap()) {
+                if missing {
+                    if is_ne {
+                        return rhs.is_not_null();
+                    } else {
+                        return rhs.is_null();
+                    }
+                } else {
+                    return BooleanChunked::full_null(PlSmallStr::EMPTY, rhs.len());
+                }
+            }
+
+            if missing {
+                arity::unary_mut_with_options(rhs, |a| broadcast_op(a, left.values()).into())
+            } else {
+                arity::unary_mut_values(rhs, |a| broadcast_op(a, left.values()).into())
+            }
+        },
+        _ => {
+            if missing {
+                arity::binary_mut_with_options(lhs, rhs, |a, b| op(a, b).into(), PlSmallStr::EMPTY)
+            } else {
+                arity::binary_mut_values(lhs, rhs, |a, b| op(a, b).into(), PlSmallStr::EMPTY)
+            }
+        },
+    }
+}
+
+#[cfg(feature = "dtype-array")]
+impl ChunkCompareEq<&ArrayChunked> for ArrayChunked {
     type Item = BooleanChunked;
     fn equal(&self, rhs: &ArrayChunked) -> BooleanChunked {
-        if self.width() != rhs.width() {
-            return BooleanChunked::full(PlSmallStr::EMPTY, false, self.len());
-        }
-        arity::binary_mut_values(
+        _array_comparison_helper(
             self,
             rhs,
-            |a, b| a.tot_eq_kernel(b).into(),
-            PlSmallStr::EMPTY,
+            TotalEqKernel::tot_eq_kernel,
+            TotalEqKernel::tot_eq_kernel_broadcast,
+            false,
+            false,
         )
     }
 
     fn equal_missing(&self, rhs: &ArrayChunked) -> BooleanChunked {
-        if self.width() != rhs.width() {
-            return BooleanChunked::full(PlSmallStr::EMPTY, false, self.len());
-        }
-        arity::binary_mut_with_options(
+        _array_comparison_helper(
             self,
             rhs,
-            |a, b| a.tot_eq_missing_kernel(b).into(),
-            PlSmallStr::EMPTY,
+            TotalEqKernel::tot_eq_missing_kernel,
+            TotalEqKernel::tot_eq_missing_kernel_broadcast,
+            true,
+            false,
         )
     }
 
     fn not_equal(&self, rhs: &ArrayChunked) -> BooleanChunked {
-        if self.width() != rhs.width() {
-            return BooleanChunked::full(PlSmallStr::EMPTY, true, self.len());
-        }
-        arity::binary_mut_values(
+        _array_comparison_helper(
             self,
             rhs,
-            |a, b| a.tot_ne_kernel(b).into(),
-            PlSmallStr::EMPTY,
+            TotalEqKernel::tot_ne_kernel,
+            TotalEqKernel::tot_ne_kernel_broadcast,
+            false,
+            true,
         )
     }
 
     fn not_equal_missing(&self, rhs: &ArrayChunked) -> Self::Item {
-        if self.width() != rhs.width() {
-            return BooleanChunked::full(PlSmallStr::EMPTY, true, self.len());
-        }
-        arity::binary_mut_with_options(
+        _array_comparison_helper(
             self,
             rhs,
-            |a, b| a.tot_ne_missing_kernel(b).into(),
-            PlSmallStr::EMPTY,
+            TotalEqKernel::tot_ne_missing_kernel,
+            TotalEqKernel::tot_ne_missing_kernel_broadcast,
+            true,
+            true,
         )
     }
-
-    // following are not implemented because gt, lt comparison of series don't make sense
-    fn gt(&self, _rhs: &ArrayChunked) -> BooleanChunked {
-        unimplemented!()
-    }
-
-    fn gt_eq(&self, _rhs: &ArrayChunked) -> BooleanChunked {
-        unimplemented!()
-    }
-
-    fn lt(&self, _rhs: &ArrayChunked) -> BooleanChunked {
-        unimplemented!()
-    }
-
-    fn lt_eq(&self, _rhs: &ArrayChunked) -> BooleanChunked {
-        unimplemented!()
-    }
 }
 
 impl Not for &BooleanChunked {
