@@ -3,6 +3,7 @@ use std::error::Error;
 use arrow::array::{Array, MutablePlString, StaticArray};
 use arrow::compute::utils::combine_validities_and;
 use polars_error::PolarsResult;
+use polars_utils::pl_str::PlSmallStr;
 
 use crate::chunked_array::metadata::MetadataProperties;
 use crate::datatypes::{ArrayCollectIterExt, ArrayFromIter};
@@ -49,7 +50,7 @@ where
     F: FnMut(&T::Array) -> Arr,
 {
     let iter = ca.downcast_iter().map(op);
-    ChunkedArray::from_chunk_iter(ca.name(), iter)
+    ChunkedArray::from_chunk_iter(ca.name().clone(), iter)
 }
 
 /// Applies a kernel that produces `Array` types.
@@ -61,9 +62,9 @@ where
     Arr: Array,
     F: FnMut(T::Array) -> Arr,
 {
-    let name = ca.name().to_owned();
+    let name = ca.name().clone();
     let iter = ca.downcast_into_iter().map(op);
-    ChunkedArray::from_chunk_iter(&name, iter)
+    ChunkedArray::from_chunk_iter(name, iter)
 }
 
 #[inline]
@@ -78,12 +79,12 @@ where
         let iter = ca
             .downcast_iter()
             .map(|arr| arr.iter().map(&mut op).collect_arr());
-        ChunkedArray::from_chunk_iter(ca.name(), iter)
+        ChunkedArray::from_chunk_iter(ca.name().clone(), iter)
     } else {
         let iter = ca
             .downcast_iter()
             .map(|arr| arr.values_iter().map(|x| op(Some(x))).collect_arr());
-        ChunkedArray::from_chunk_iter(ca.name(), iter)
+        ChunkedArray::from_chunk_iter(ca.name().clone(), iter)
     }
 }
 
@@ -101,7 +102,7 @@ where
     let iter = ca
         .downcast_iter()
         .map(|arr| arr.iter().map(&mut op).try_collect_arr());
-    ChunkedArray::try_from_chunk_iter(ca.name(), iter)
+    ChunkedArray::try_from_chunk_iter(ca.name().clone(), iter)
 }
 
 #[inline]
@@ -114,7 +115,7 @@ where
 {
     if ca.null_count() == ca.len() {
         let arr = V::Array::full_null(ca.len(), V::get_dtype().to_arrow(CompatLevel::newest()));
-        return ChunkedArray::with_chunk(ca.name(), arr);
+        return ChunkedArray::with_chunk(ca.name().clone(), arr);
     }
 
     let iter = ca.downcast_iter().map(|arr| {
@@ -122,7 +123,7 @@ where
         let arr: V::Array = arr.values_iter().map(&mut op).collect_arr();
         arr.with_validity_typed(validity)
     });
-    ChunkedArray::from_chunk_iter(ca.name(), iter)
+    ChunkedArray::from_chunk_iter(ca.name().clone(), iter)
 }
 
 #[inline]
@@ -138,7 +139,7 @@ where
 {
     if ca.null_count() == ca.len() {
         let arr = V::Array::full_null(ca.len(), V::get_dtype().to_arrow(CompatLevel::newest()));
-        return Ok(ChunkedArray::with_chunk(ca.name(), arr));
+        return Ok(ChunkedArray::with_chunk(ca.name().clone(), arr));
     }
 
     let iter = ca.downcast_iter().map(|arr| {
@@ -146,7 +147,7 @@ where
         let arr: V::Array = arr.values_iter().map(&mut op).try_collect_arr()?;
         Ok(arr.with_validity_typed(validity))
     });
-    ChunkedArray::try_from_chunk_iter(ca.name(), iter)
+    ChunkedArray::try_from_chunk_iter(ca.name().clone(), iter)
 }
 
 /// Applies a kernel that produces `Array` types.
@@ -164,7 +165,7 @@ where
     let iter = ca
         .downcast_iter()
         .map(|arr| op(arr).with_validity_typed(arr.validity().cloned()));
-    ChunkedArray::from_chunk_iter(ca.name(), iter)
+    ChunkedArray::from_chunk_iter(ca.name().clone(), iter)
 }
 
 /// Applies a kernel that produces `Array` types.
@@ -176,7 +177,7 @@ where
     Arr: Array + StaticArray,
     F: FnMut(&T::Array) -> Arr,
 {
-    ChunkedArray::from_chunk_iter(ca.name(), ca.downcast_iter().map(op))
+    ChunkedArray::from_chunk_iter(ca.name().clone(), ca.downcast_iter().map(op))
 }
 
 #[inline]
@@ -191,7 +192,7 @@ where
     F: FnMut(&T::Array) -> Result<Arr, E>,
     E: Error,
 {
-    ChunkedArray::try_from_chunk_iter(ca.name(), ca.downcast_iter().map(op))
+    ChunkedArray::try_from_chunk_iter(ca.name().clone(), ca.downcast_iter().map(op))
 }
 
 #[inline]
@@ -220,7 +221,7 @@ where
                 .map(|(lhs_opt_val, rhs_opt_val)| op(lhs_opt_val, rhs_opt_val));
             element_iter.collect_arr()
         });
-    ChunkedArray::from_chunk_iter(lhs.name(), iter)
+    ChunkedArray::from_chunk_iter(lhs.name().clone(), iter)
 }
 
 #[inline]
@@ -297,7 +298,7 @@ where
                 .map(|(lhs_opt_val, rhs_opt_val)| op(lhs_opt_val, rhs_opt_val));
             element_iter.try_collect_arr()
         });
-    ChunkedArray::try_from_chunk_iter(lhs.name(), iter)
+    ChunkedArray::try_from_chunk_iter(lhs.name().clone(), iter)
 }
 
 #[inline]
@@ -317,7 +318,7 @@ where
         let len = lhs.len().min(rhs.len());
         let arr = V::Array::full_null(len, V::get_dtype().to_arrow(CompatLevel::newest()));
 
-        return ChunkedArray::with_chunk(lhs.name(), arr);
+        return ChunkedArray::with_chunk(lhs.name().clone(), arr);
     }
 
     let (lhs, rhs) = align_chunks_binary(lhs, rhs);
@@ -336,7 +337,7 @@ where
             let array: V::Array = element_iter.collect_arr();
             array.with_validity_typed(validity)
         });
-    ChunkedArray::from_chunk_iter(lhs.name(), iter)
+    ChunkedArray::from_chunk_iter(lhs.name().clone(), iter)
 }
 
 /// Apply elementwise binary function which produces string, amortising allocations.
@@ -373,7 +374,7 @@ where
                 });
             mutarr.freeze()
         });
-    ChunkedArray::from_chunk_iter(lhs.name(), iter)
+    ChunkedArray::from_chunk_iter(lhs.name().clone(), iter)
 }
 
 /// Applies a kernel that produces `Array` types.
@@ -385,7 +386,7 @@ pub fn binary_mut_values<T, U, V, F, Arr>(
     lhs: &ChunkedArray<T>,
     rhs: &ChunkedArray<U>,
     mut op: F,
-    name: &str,
+    name: PlSmallStr,
 ) -> ChunkedArray<V>
 where
     T: PolarsDataType,
@@ -413,7 +414,7 @@ pub fn binary_mut_with_options<T, U, V, F, Arr>(
     lhs: &ChunkedArray<T>,
     rhs: &ChunkedArray<U>,
     mut op: F,
-    name: &str,
+    name: PlSmallStr,
 ) -> ChunkedArray<V>
 where
     T: PolarsDataType,
@@ -435,7 +436,7 @@ pub fn try_binary_mut_with_options<T, U, V, F, Arr, E>(
     lhs: &ChunkedArray<T>,
     rhs: &ChunkedArray<U>,
     mut op: F,
-    name: &str,
+    name: PlSmallStr,
 ) -> Result<ChunkedArray<V>, E>
 where
     T: PolarsDataType,
@@ -466,7 +467,7 @@ where
     Arr: Array,
     F: FnMut(&T::Array, &U::Array) -> Arr,
 {
-    binary_mut_with_options(lhs, rhs, op, lhs.name())
+    binary_mut_with_options(lhs, rhs, op, lhs.name().clone())
 }
 
 /// Applies a kernel that produces `Array` types.
@@ -482,13 +483,13 @@ where
     Arr: Array,
     F: FnMut(L::Array, R::Array) -> Arr,
 {
-    let name = lhs.name().to_owned();
+    let name = lhs.name().clone();
     let (lhs, rhs) = align_chunks_binary_owned(lhs, rhs);
     let iter = lhs
         .downcast_into_iter()
         .zip(rhs.downcast_into_iter())
         .map(|(lhs_arr, rhs_arr)| op(lhs_arr, rhs_arr));
-    ChunkedArray::from_chunk_iter(&name, iter)
+    ChunkedArray::from_chunk_iter(name, iter)
 }
 
 /// Applies a kernel that produces `Array` types.
@@ -510,7 +511,7 @@ where
         .downcast_iter()
         .zip(rhs.downcast_iter())
         .map(|(lhs_arr, rhs_arr)| op(lhs_arr, rhs_arr));
-    ChunkedArray::try_from_chunk_iter(lhs.name(), iter)
+    ChunkedArray::try_from_chunk_iter(lhs.name().clone(), iter)
 }
 
 /// Applies a kernel that produces `ArrayRef` of the same type.
@@ -566,7 +567,7 @@ where
         .zip(rhs.downcast_iter())
         .map(|(lhs_arr, rhs_arr)| op(lhs_arr, rhs_arr))
         .collect::<Vec<_>>();
-    Series::try_from((lhs.name(), chunks))
+    Series::try_from((lhs.name().clone(), chunks))
 }
 
 /// Applies a kernel that produces `ArrayRef` of the same type.
@@ -636,7 +637,7 @@ where
             );
             element_iter.try_collect_arr()
         });
-    ChunkedArray::try_from_chunk_iter(ca1.name(), iter)
+    ChunkedArray::try_from_chunk_iter(ca1.name().clone(), iter)
 }
 
 #[inline]
@@ -677,7 +678,7 @@ where
             );
             element_iter.collect_arr()
         });
-    ChunkedArray::from_chunk_iter(ca1.name(), iter)
+    ChunkedArray::from_chunk_iter(ca1.name().clone(), iter)
 }
 
 pub fn broadcast_binary_elementwise<T, U, V, F>(
@@ -697,7 +698,7 @@ where
     match (lhs.len(), rhs.len()) {
         (1, _) => {
             let a = unsafe { lhs.get_unchecked(0) };
-            unary_elementwise(rhs, |b| op(a.clone(), b)).with_name(lhs.name())
+            unary_elementwise(rhs, |b| op(a.clone(), b)).with_name(lhs.name().clone())
         },
         (_, 1) => {
             let b = unsafe { rhs.get_unchecked(0) };
@@ -722,7 +723,7 @@ where
     match (lhs.len(), rhs.len()) {
         (1, _) => {
             let a = unsafe { lhs.get_unchecked(0) };
-            Ok(try_unary_elementwise(rhs, |b| op(a.clone(), b))?.with_name(lhs.name()))
+            Ok(try_unary_elementwise(rhs, |b| op(a.clone(), b))?.with_name(lhs.name().clone()))
         },
         (_, 1) => {
             let b = unsafe { rhs.get_unchecked(0) };
@@ -750,13 +751,13 @@ where
         let len = if min == 1 { max } else { min };
         let arr = V::Array::full_null(len, V::get_dtype().to_arrow(CompatLevel::newest()));
 
-        return ChunkedArray::with_chunk(lhs.name(), arr);
+        return ChunkedArray::with_chunk(lhs.name().clone(), arr);
     }
 
     match (lhs.len(), rhs.len()) {
         (1, _) => {
             let a = unsafe { lhs.value_unchecked(0) };
-            unary_elementwise_values(rhs, |b| op(a.clone(), b)).with_name(lhs.name())
+            unary_elementwise_values(rhs, |b| op(a.clone(), b)).with_name(lhs.name().clone())
         },
         (_, 1) => {
             let b = unsafe { rhs.value_unchecked(0) };
@@ -793,7 +794,7 @@ where
                         lhs.len(),
                         O::get_dtype().to_arrow(CompatLevel::newest()),
                     );
-                    ChunkedArray::<O>::with_chunk(lhs.name(), arr)
+                    ChunkedArray::<O>::with_chunk(lhs.name().clone(), arr)
                 },
                 Some(rhs) => unary_kernel(lhs, |arr| rhs_broadcast_kernel(arr, rhs.clone())),
             }
@@ -806,14 +807,14 @@ where
                         rhs.len(),
                         O::get_dtype().to_arrow(CompatLevel::newest()),
                     );
-                    ChunkedArray::<O>::with_chunk(lhs.name(), arr)
+                    ChunkedArray::<O>::with_chunk(lhs.name().clone(), arr)
                 },
                 Some(lhs) => unary_kernel(rhs, |arr| lhs_broadcast_kernel(lhs.clone(), arr)),
             }
         },
         _ => panic!("Cannot apply operation on arrays of different lengths"),
     };
-    out.with_name(name)
+    out.with_name(name.clone())
 }
 
 pub fn apply_binary_kernel_broadcast_owned<L, R, O, K, LK, RK>(
@@ -843,7 +844,7 @@ where
                         lhs.len(),
                         O::get_dtype().to_arrow(CompatLevel::newest()),
                     );
-                    ChunkedArray::<O>::with_chunk(lhs.name(), arr)
+                    ChunkedArray::<O>::with_chunk(lhs.name().clone(), arr)
                 },
                 Some(rhs) => unary_kernel_owned(lhs, |arr| rhs_broadcast_kernel(arr, rhs.clone())),
             }
@@ -856,12 +857,12 @@ where
                         rhs.len(),
                         O::get_dtype().to_arrow(CompatLevel::newest()),
                     );
-                    ChunkedArray::<O>::with_chunk(lhs.name(), arr)
+                    ChunkedArray::<O>::with_chunk(lhs.name().clone(), arr)
                 },
                 Some(lhs) => unary_kernel_owned(rhs, |arr| lhs_broadcast_kernel(lhs.clone(), arr)),
             }
         },
         _ => panic!("Cannot apply operation on arrays of different lengths"),
     };
-    out.with_name(&name)
+    out.with_name(name)
 }
