@@ -21,7 +21,7 @@ pub type IdxType = UInt32Type;
 #[cfg(feature = "bigidx")]
 pub type IdxType = UInt64Type;
 
-pub use smartstring::alias::String as SmartString;
+pub use polars_utils::pl_str::PlSmallStr;
 
 /// This hashmap uses an IdHasher
 pub type PlIdHashMap<K, V> = hashbrown::HashMap<K, V, IdBuildHasher>;
