@@ -3,7 +3,6 @@ use std::ops::Deref;
 
 use arrow::offset::OffsetsBuffer;
 use polars_utils::idx_vec::IdxVec;
-use polars_utils::sync::SyncPtr;
 use rayon::iter::plumbing::UnindexedConsumer;
 use rayon::prelude::*;
 
@@ -46,61 +45,6 @@ impl From<Vec<IdxItem>> for GroupsIdx {
     }
 }
 
-impl From<Vec<(Vec<IdxSize>, Vec<IdxVec>)>> for GroupsIdx {
-    fn from(v: Vec<(Vec<IdxSize>, Vec<IdxVec>)>) -> Self {
-        // we have got the hash tables so we can determine the final
-        let cap = v.iter().map(|v| v.0.len()).sum::<usize>();
-        let offsets = v
-            .iter()
-            .scan(0_usize, |acc, v| {
-                let out = *acc;
-                *acc += v.0.len();
-                Some(out)
-            })
-            .collect::<Vec<_>>();
-        let mut global_first = Vec::with_capacity(cap);
-        let global_first_ptr = unsafe { SyncPtr::new(global_first.as_mut_ptr()) };
-        let mut global_all = Vec::with_capacity(cap);
-        let global_all_ptr = unsafe { SyncPtr::new(global_all.as_mut_ptr()) };
-
-        POOL.install(|| {
-            v.into_par_iter().zip(offsets).for_each(
-                |((local_first_vals, mut local_all_vals), offset)| unsafe {
-                    let global_first: *mut IdxSize = global_first_ptr.get();
-                    let global_all: *mut IdxVec = global_all_ptr.get();
-                    let global_first = global_first.add(offset);
-                    let global_all = global_all.add(offset);
-
-                    std::ptr::copy_nonoverlapping(
-                        local_first_vals.as_ptr(),
-                        global_first,
-                        local_first_vals.len(),
-                    );
-                    std::ptr::copy_nonoverlapping(
-                        local_all_vals.as_ptr(),
-                        global_all,
-                        local_all_vals.len(),
-                    );
-                    // local_all_vals: Vec<Vec<IdxSize>>
-                    // we just copied the contents: Vec<IdxSize> to a new buffer
-                    // now, we want to free the outer vec, without freeing
-                    // the inner vecs as they are moved, so we set the len to 0
-                    local_all_vals.set_len(0);
-                },
-            );
-        });
-        unsafe {
-            global_all.set_len(cap);
-            global_first.set_len(cap);
-        }
-        GroupsIdx {
-            sorted: false,
-            first: global_first,
-            all: global_all,
-        }
-    }
-}
-
 impl From<Vec<Vec<IdxItem>>> for GroupsIdx {
     fn from(v: Vec<Vec<IdxItem>>) -> Self {
         // single threaded flatten: 10% faster than `iter().flatten().collect()
@@ -149,6 +93,9 @@ impl GroupsIdx {
     }
 
     pub fn sort(&mut self) {
+        if self.sorted {
+            return;
+        }
         let mut idx = 0;
         let first = std::mem::take(&mut self.first);
         // store index and values so that we can sort those
@@ -542,7 +489,7 @@ impl GroupsProxy {
         }
     }
 
-    pub fn slice(&self, offset: i64, len: usize) -> SlicedGroups {
+    pub fn slice(&self, offset: i64, len: usize) -> SlicedGroups<'_> {
         // SAFETY:
         // we create new `Vec`s from the sliced groups. But we wrap them in ManuallyDrop
         // so that we never call drop on them.
