@@ -20,6 +20,8 @@ pub trait SchemaExt {
     fn iter_fields(&self) -> impl ExactSizeIterator<Item = Field> + '_;
 
     fn to_supertype(&mut self, other: &Schema) -> PolarsResult<bool>;
+
+    fn materialize_unknown_dtypes(&self) -> PolarsResult<Schema>;
 }
 
 impl SchemaExt for Schema {
@@ -88,6 +90,13 @@ impl SchemaExt for Schema {
         }
         Ok(changed)
     }
+
+    /// Materialize all unknown dtypes in this schema.
+    fn materialize_unknown_dtypes(&self) -> PolarsResult<Schema> {
+        self.iter()
+            .map(|(name, dtype)| Ok((name.clone(), dtype.materialize_unknown()?)))
+            .collect()
+    }
 }
 
 pub trait SchemaNamesAndDtypes {
