@@ -2,8 +2,7 @@ use std::fmt::Write;
 
 use arrow::bitmap::MutableBitmap;
 
-#[cfg(feature = "dtype-categorical")]
-use crate::chunked_array::cast::CastOptions;
+use crate::chunked_array::builder::{get_list_builder, AnonymousOwnedListBuilder};
 #[cfg(feature = "object")]
 use crate::chunked_array::object::registry::ObjectRegistry;
 use crate::prelude::*;
@@ -146,9 +145,9 @@ impl Series {
             #[cfg(feature = "dtype-duration")]
             DataType::Duration(tu) => any_values_to_duration(values, *tu, strict)?.into_series(),
             #[cfg(feature = "dtype-categorical")]
-            dt @ (DataType::Categorical(_, _) | DataType::Enum(_, _)) => {
-                any_values_to_categorical(values, dt, strict)?
-            },
+            dt @ DataType::Categorical(_, _) => any_values_to_categorical(values, dt, strict)?,
+            #[cfg(feature = "dtype-categorical")]
+            dt @ DataType::Enum(_, _) => any_values_to_enum(values, dt, strict)?,
             #[cfg(feature = "dtype-decimal")]
             DataType::Decimal(precision, scale) => {
                 any_values_to_decimal(values, *precision, *scale, strict)?.into_series()
@@ -398,6 +397,7 @@ fn any_values_to_datetime(
     for av in values {
         match av {
             AnyValue::Datetime(i, tu, _) if *tu == time_unit => builder.append_value(*i),
+            AnyValue::DatetimeOwned(i, tu, _) if *tu == time_unit => builder.append_value(*i),
             AnyValue::Null => builder.append_null(),
             av => {
                 if strict {
@@ -405,6 +405,7 @@ fn any_values_to_datetime(
                 }
                 match av.cast(&target_dtype) {
                     AnyValue::Datetime(i, _, _) => builder.append_value(i),
+                    AnyValue::DatetimeOwned(i, _, _) => builder.append_value(i),
                     _ => builder.append_null(),
                 }
             },
@@ -445,14 +446,91 @@ fn any_values_to_categorical(
     dtype: &DataType,
     strict: bool,
 ) -> PolarsResult<Series> {
-    // TODO: Handle AnyValues of type Categorical/Enum.
-    // TODO: Avoid materializing to String before casting to Categorical/Enum.
-    let ca = any_values_to_string(values, strict)?;
-    if strict {
-        ca.into_series().strict_cast(dtype)
-    } else {
-        ca.cast_with_options(dtype, CastOptions::NonStrict)
+    let ordering = match dtype {
+        DataType::Categorical(_, ordering) => ordering,
+        _ => panic!("any_values_to_categorical with dtype={dtype:?}"),
+    };
+
+    let mut builder = CategoricalChunkedBuilder::new(PlSmallStr::EMPTY, values.len(), *ordering);
+
+    let mut owned = String::new(); // Amortize allocations.
+    for av in values {
+        match av {
+            AnyValue::String(s) => builder.append_value(s),
+            AnyValue::StringOwned(s) => builder.append_value(s),
+
+            AnyValue::Enum(s, rev, _) => builder.append_value(rev.get(*s)),
+            AnyValue::EnumOwned(s, rev, _) => builder.append_value(rev.get(*s)),
+
+            AnyValue::Categorical(s, rev, _) => builder.append_value(rev.get(*s)),
+            AnyValue::CategoricalOwned(s, rev, _) => builder.append_value(rev.get(*s)),
+
+            AnyValue::Binary(_) | AnyValue::BinaryOwned(_) if !strict => builder.append_null(),
+            AnyValue::Null => builder.append_null(),
+
+            av => {
+                if strict {
+                    return Err(invalid_value_error(&DataType::String, av));
+                }
+
+                owned.clear();
+                write!(owned, "{av}").unwrap();
+                builder.append_value(&owned);
+            },
+        }
     }
+
+    let ca = builder.finish();
+
+    Ok(ca.into_series())
+}
+
+#[cfg(feature = "dtype-categorical")]
+fn any_values_to_enum(values: &[AnyValue], dtype: &DataType, strict: bool) -> PolarsResult<Series> {
+    use self::enum_::EnumChunkedBuilder;
+
+    let (rev, ordering) = match dtype {
+        DataType::Enum(rev, ordering) => (rev.clone(), ordering),
+        _ => panic!("any_values_to_categorical with dtype={dtype:?}"),
+    };
+
+    let Some(rev) = rev else {
+        polars_bail!(nyi = "Not yet possible to create enum series without a rev-map");
+    };
+
+    let mut builder =
+        EnumChunkedBuilder::new(PlSmallStr::EMPTY, values.len(), rev, *ordering, strict);
+
+    let mut owned = String::new(); // Amortize allocations.
+    for av in values {
+        match av {
+            AnyValue::String(s) => builder.append_str(s)?,
+            AnyValue::StringOwned(s) => builder.append_str(s)?,
+
+            AnyValue::Enum(s, rev, _) => builder.append_enum(*s, rev)?,
+            AnyValue::EnumOwned(s, rev, _) => builder.append_enum(*s, rev)?,
+
+            AnyValue::Categorical(s, rev, _) => builder.append_str(rev.get(*s))?,
+            AnyValue::CategoricalOwned(s, rev, _) => builder.append_str(rev.get(*s))?,
+
+            AnyValue::Binary(_) | AnyValue::BinaryOwned(_) if !strict => builder.append_null(),
+            AnyValue::Null => builder.append_null(),
+
+            av => {
+                if strict {
+                    return Err(invalid_value_error(&DataType::String, av));
+                }
+
+                owned.clear();
+                write!(owned, "{av}").unwrap();
+                builder.append_str(&owned)?
+            },
+        };
+    }
+
+    let ca = builder.finish();
+
+    Ok(ca.into_series())
 }
 
 #[cfg(feature = "dtype-decimal")]
@@ -528,76 +606,96 @@ fn any_values_to_list(
     inner_type: &DataType,
     strict: bool,
 ) -> PolarsResult<ListChunked> {
-    let it = match inner_type {
-        // Structs don't support empty fields yet.
-        // We must ensure the data-types match what we do physical
-        #[cfg(feature = "dtype-struct")]
-        DataType::Struct(fields) if fields.is_empty() => {
-            DataType::Struct(vec![Field::new(PlSmallStr::EMPTY, DataType::Null)])
-        },
-        _ => inner_type.clone(),
-    };
-    let target_dtype = DataType::List(Box::new(it));
+    // GB:
+    // Lord forgive for the sins I have committed in this function. The amount of strange
+    // exceptions that need to happen for this to work are insane and I feel like I am going crazy.
+    //
+    // This function is essentially a copy of the `<ListChunked as FromIterator>` where it does not
+    // sample the datatype from the first element and instead we give it explicitly. This allows
+    // this function to properly assign a datatype if `avs` starts with a `null` value. Previously,
+    // this was solved by assigning the `dtype` again afterwards, but why? We should not link the
+    // implementation of these functions. We still need to assign the dtype of the ListArray and
+    // such, anyways.
+    //
+    // Then, `collect_ca_with_dtype` does not possess the necessary exceptions shown in this
+    // function to use that. I have tried adding the exceptions there and it broke other things. I
+    // really do feel like this is the simplest solution.
 
-    // This is handled downstream. The builder will choose the first non-null type.
     let mut valid = true;
-    #[allow(unused_mut)]
-    let mut out: ListChunked = if inner_type == &DataType::Null {
-        avs.iter()
-            .map(|av| match av {
-                AnyValue::List(b) => Some(b.clone()),
-                AnyValue::Null => None,
-                _ => {
-                    valid = false;
-                    None
+    let capacity = avs.len();
+
+    let ca = match inner_type {
+        // AnyValues with empty lists in python can create
+        // Series of an unknown dtype.
+        // We use the anonymousbuilder without a dtype
+        // the empty arrays is then not added (we add an extra offset instead)
+        // the next non-empty series then must have the correct dtype.
+        DataType::Null => {
+            let mut builder = AnonymousOwnedListBuilder::new(PlSmallStr::EMPTY, capacity, None);
+            for av in avs {
+                match av {
+                    AnyValue::List(b) => builder.append_series(b)?,
+                    AnyValue::Null => builder.append_null(),
+                    _ => {
+                        valid = false;
+                        builder.append_null();
+                    },
+                }
+            }
+            builder.finish()
+        },
+
+        #[cfg(feature = "object")]
+        DataType::Object(_, _) => polars_bail!(nyi = "Nested object types"),
+
+        _ => {
+            let list_inner_type = match inner_type {
+                // Categoricals may not have a revmap yet. We just give them an empty one here and
+                // the list builder takes care of the rest.
+                #[cfg(feature = "dtype-categorical")]
+                DataType::Categorical(None, ordering) => {
+                    DataType::Categorical(Some(Arc::new(RevMapping::default())), *ordering)
                 },
-            })
-            .collect_trusted()
-    }
-    // Make sure that wrongly inferred AnyValues don't deviate from the datatype.
-    else {
-        avs.iter()
-            .map(|av| match av {
-                AnyValue::List(b) => {
-                    if b.dtype() == inner_type {
-                        Some(b.clone())
-                    } else {
-                        match b.cast(inner_type) {
-                            Ok(out) => {
-                                if out.null_count() != b.null_count() {
-                                    valid = !strict;
-                                }
-                                Some(out)
-                            },
-                            Err(_) => {
+
+                _ => inner_type.clone(),
+            };
+
+            let mut builder =
+                get_list_builder(&list_inner_type, capacity * 5, capacity, PlSmallStr::EMPTY);
+
+            for av in avs {
+                match av {
+                    AnyValue::List(b) => match b.cast(inner_type) {
+                        Ok(casted) => {
+                            if casted.null_count() != b.null_count() {
                                 valid = !strict;
-                                Some(Series::full_null(b.name().clone(), b.len(), inner_type))
-                            },
-                        }
-                    }
-                },
-                AnyValue::Null => None,
-                _ => {
-                    valid = false;
-                    None
-                },
-            })
-            .collect_trusted()
+                            }
+                            builder.append_series(&casted)?;
+                        },
+                        Err(_) => {
+                            valid = false;
+                            for _ in 0..b.len() {
+                                builder.append_null();
+                            }
+                        },
+                    },
+                    AnyValue::Null => builder.append_null(),
+                    _ => {
+                        valid = false;
+                        builder.append_null()
+                    },
+                }
+            }
+
+            builder.finish()
+        },
     };
 
     if strict && !valid {
-        polars_bail!(SchemaMismatch: "unexpected value while building Series of type {:?}", target_dtype);
+        polars_bail!(SchemaMismatch: "unexpected value while building Series of type {:?}", DataType::List(Box::new(inner_type.clone())));
     }
 
-    // Ensure the logical type is correct for nested types.
-    #[cfg(feature = "dtype-struct")]
-    if !matches!(inner_type, DataType::Null) && out.inner_dtype().is_nested() {
-        unsafe {
-            out.set_dtype(target_dtype.clone());
-        };
-    }
-
-    Ok(out)
+    Ok(ca)
 }
 
 #[cfg(feature = "dtype-array")]
@@ -727,7 +825,9 @@ fn any_values_to_struct(
 ) -> PolarsResult<Series> {
     // Fast path for structs with no fields.
     if fields.is_empty() {
-        return Ok(StructChunked::full_null(PlSmallStr::EMPTY, values.len()).into_series());
+        return Ok(
+            StructChunked::from_series(PlSmallStr::EMPTY, values.len(), [].iter())?.into_series(),
+        );
     }
 
     // The physical series fields of the struct.
@@ -768,7 +868,8 @@ fn any_values_to_struct(
         series_fields.push(s)
     }
 
-    let mut out = StructChunked::from_series(PlSmallStr::EMPTY, series_fields.iter())?;
+    let mut out =
+        StructChunked::from_series(PlSmallStr::EMPTY, values.len(), series_fields.iter())?;
     if has_outer_validity {
         let mut validity = MutableBitmap::new();
         validity.extend_constant(values.len(), true);
