@@ -4,8 +4,8 @@ use crate::prelude::*;
 use crate::series::arithmetic::coerce_lhs_rhs;
 use crate::series::nulls::replace_non_null;
 
-macro_rules! impl_compare {
-    ($self:expr, $rhs:expr, $method:ident, $struct_function:expr) => {{
+macro_rules! impl_eq_compare {
+    ($self:expr, $rhs:expr, $method:ident) => {{
         use DataType::*;
         let (lhs, rhs) = ($self, $rhs);
         validate_types(lhs.dtype(), rhs.dtype())?;
@@ -70,14 +70,7 @@ macro_rules! impl_compare {
             #[cfg(feature = "dtype-array")]
             Array(_, _) => lhs.array().unwrap().$method(rhs.array().unwrap()),
             #[cfg(feature = "dtype-struct")]
-            Struct(_) => {
-                let lhs = lhs
-                .struct_()
-                .unwrap();
-                let rhs = rhs.struct_().unwrap();
-
-                $struct_function(lhs, rhs)?
-            },
+            Struct(_) => lhs.struct_().unwrap().$method(rhs.struct_().unwrap()),
             #[cfg(feature = "dtype-decimal")]
             Decimal(_, s1) => {
                 let DataType::Decimal(_, s2) = rhs.dtype() else {
@@ -96,14 +89,108 @@ macro_rules! impl_compare {
     }};
 }
 
-#[cfg(feature = "dtype-struct")]
-fn raise_struct(_a: &StructChunked, _b: &StructChunked) -> PolarsResult<BooleanChunked> {
-    polars_bail!(InvalidOperation: "order comparison not support for struct dtype")
+macro_rules! bail_invalid_ineq {
+    ($lhs:expr, $rhs:expr, $op:literal) => {
+        polars_bail!(
+            InvalidOperation: "cannot perform '{}' comparison between series '{}' of dtype: {} and series '{}' of dtype: {}",
+            $op,
+            $lhs.name(), $lhs.dtype(),
+            $rhs.name(), $rhs.dtype(),
+        )
+    };
 }
 
-#[cfg(not(feature = "dtype-struct"))]
-fn raise_struct(_a: &(), _b: &()) -> PolarsResult<BooleanChunked> {
-    unimplemented!()
+macro_rules! impl_ineq_compare {
+    ($self:expr, $rhs:expr, $method:ident, $op:literal) => {{
+        use DataType::*;
+        let (lhs, rhs) = ($self, $rhs);
+        validate_types(lhs.dtype(), rhs.dtype())?;
+
+        polars_ensure!(
+            lhs.len() == rhs.len() ||
+
+            // Broadcast
+            lhs.len() == 1 ||
+            rhs.len() == 1,
+            ShapeMismatch:
+                "could not perform '{}' comparison between series '{}' of length: {} and series '{}' of length: {}, because they have different lengths",
+            $op,
+            lhs.name(), lhs.len(),
+            rhs.name(), rhs.len()
+        );
+
+        #[cfg(feature = "dtype-categorical")]
+        match (lhs.dtype(), rhs.dtype()) {
+            (Categorical(_, _) | Enum(_, _), Categorical(_, _) | Enum(_, _)) => {
+                return Ok(lhs
+                    .categorical()
+                    .unwrap()
+                    .$method(rhs.categorical().unwrap())?
+                    .with_name(lhs.name().clone()));
+            },
+            (Categorical(_, _) | Enum(_, _), String) => {
+                return Ok(lhs
+                    .categorical()
+                    .unwrap()
+                    .$method(rhs.str().unwrap())?
+                    .with_name(lhs.name().clone()));
+            },
+            (String, Categorical(_, _) | Enum(_, _)) => {
+                return Ok(rhs
+                    .categorical()
+                    .unwrap()
+                    .$method(lhs.str().unwrap())?
+                    .with_name(lhs.name().clone()));
+            },
+            _ => (),
+        };
+
+        let (lhs, rhs) = coerce_lhs_rhs(lhs, rhs).map_err(|_|
+            polars_err!(
+                SchemaMismatch: "could not evaluate '{}' comparison between series '{}' of dtype: {} and series '{}' of dtype: {}",
+                $op,
+                lhs.name(), lhs.dtype(),
+                rhs.name(), rhs.dtype()
+            )
+        )?;
+        let lhs = lhs.to_physical_repr();
+        let rhs = rhs.to_physical_repr();
+        let mut out = match lhs.dtype() {
+            Null => lhs.null().unwrap().$method(rhs.null().unwrap()),
+            Boolean => lhs.bool().unwrap().$method(rhs.bool().unwrap()),
+            String => lhs.str().unwrap().$method(rhs.str().unwrap()),
+            Binary => lhs.binary().unwrap().$method(rhs.binary().unwrap()),
+            UInt8 => lhs.u8().unwrap().$method(rhs.u8().unwrap()),
+            UInt16 => lhs.u16().unwrap().$method(rhs.u16().unwrap()),
+            UInt32 => lhs.u32().unwrap().$method(rhs.u32().unwrap()),
+            UInt64 => lhs.u64().unwrap().$method(rhs.u64().unwrap()),
+            Int8 => lhs.i8().unwrap().$method(rhs.i8().unwrap()),
+            Int16 => lhs.i16().unwrap().$method(rhs.i16().unwrap()),
+            Int32 => lhs.i32().unwrap().$method(rhs.i32().unwrap()),
+            Int64 => lhs.i64().unwrap().$method(rhs.i64().unwrap()),
+            Float32 => lhs.f32().unwrap().$method(rhs.f32().unwrap()),
+            Float64 => lhs.f64().unwrap().$method(rhs.f64().unwrap()),
+            List(_) => bail_invalid_ineq!(lhs, rhs, $op),
+            #[cfg(feature = "dtype-array")]
+            Array(_, _) => bail_invalid_ineq!(lhs, rhs, $op),
+            #[cfg(feature = "dtype-struct")]
+            Struct(_) => bail_invalid_ineq!(lhs, rhs, $op),
+            #[cfg(feature = "dtype-decimal")]
+            Decimal(_, s1) => {
+                let DataType::Decimal(_, s2) = rhs.dtype() else {
+                    unreachable!()
+                };
+                let scale = s1.max(s2).unwrap();
+                let lhs = lhs.decimal().unwrap().to_scale(scale).unwrap();
+                let rhs = rhs.decimal().unwrap().to_scale(scale).unwrap();
+                lhs.0.$method(&rhs.0)
+            },
+
+            dt => polars_bail!(InvalidOperation: "could not apply comparison on series of dtype '{}; operand names: '{}', '{}'", dt, lhs.name(), rhs.name()),
+        };
+        out.rename(lhs.name().clone());
+        PolarsResult::Ok(out)
+    }};
 }
 
 fn validate_types(left: &DataType, right: &DataType) -> PolarsResult<()> {
@@ -124,74 +211,61 @@ fn validate_types(left: &DataType, right: &DataType) -> PolarsResult<()> {
     Ok(())
 }
 
-impl ChunkCompare<&Series> for Series {
+impl ChunkCompareEq<&Series> for Series {
     type Item = PolarsResult<BooleanChunked>;
 
     /// Create a boolean mask by checking for equality.
-    fn equal(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
-        impl_compare!(self, rhs, equal, |a: &StructChunked, b: &StructChunked| {
-            PolarsResult::Ok(a.equal(b))
-        })
+    fn equal(&self, rhs: &Series) -> Self::Item {
+        impl_eq_compare!(self, rhs, equal)
     }
 
     /// Create a boolean mask by checking for equality.
-    fn equal_missing(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
-        impl_compare!(
-            self,
-            rhs,
-            equal_missing,
-            |a: &StructChunked, b: &StructChunked| PolarsResult::Ok(a.equal_missing(b))
-        )
+    fn equal_missing(&self, rhs: &Series) -> Self::Item {
+        impl_eq_compare!(self, rhs, equal_missing)
     }
 
     /// Create a boolean mask by checking for inequality.
-    fn not_equal(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
-        impl_compare!(
-            self,
-            rhs,
-            not_equal,
-            |a: &StructChunked, b: &StructChunked| PolarsResult::Ok(a.not_equal(b))
-        )
+    fn not_equal(&self, rhs: &Series) -> Self::Item {
+        impl_eq_compare!(self, rhs, not_equal)
     }
 
     /// Create a boolean mask by checking for inequality.
-    fn not_equal_missing(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
-        impl_compare!(
-            self,
-            rhs,
-            not_equal_missing,
-            |a: &StructChunked, b: &StructChunked| PolarsResult::Ok(a.not_equal_missing(b))
-        )
-    }
-
-    /// Create a boolean mask by checking if self > rhs.
-    fn gt(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
-        impl_compare!(self, rhs, gt, raise_struct)
-    }
-
-    /// Create a boolean mask by checking if self >= rhs.
-    fn gt_eq(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
-        impl_compare!(self, rhs, gt_eq, raise_struct)
-    }
-
-    /// Create a boolean mask by checking if self < rhs.
-    fn lt(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
-        impl_compare!(self, rhs, lt, raise_struct)
-    }
-
-    /// Create a boolean mask by checking if self <= rhs.
-    fn lt_eq(&self, rhs: &Series) -> PolarsResult<BooleanChunked> {
-        impl_compare!(self, rhs, lt_eq, raise_struct)
+    fn not_equal_missing(&self, rhs: &Series) -> Self::Item {
+        impl_eq_compare!(self, rhs, not_equal_missing)
     }
 }
 
-impl<Rhs> ChunkCompare<Rhs> for Series
+impl ChunkCompareIneq<&Series> for Series {
+    type Item = PolarsResult<BooleanChunked>;
+
+    /// Create a boolean mask by checking if self > rhs.
+    fn gt(&self, rhs: &Series) -> Self::Item {
+        impl_ineq_compare!(self, rhs, gt, ">")
+    }
+
+    /// Create a boolean mask by checking if self >= rhs.
+    fn gt_eq(&self, rhs: &Series) -> Self::Item {
+        impl_ineq_compare!(self, rhs, gt_eq, ">=")
+    }
+
+    /// Create a boolean mask by checking if self < rhs.
+    fn lt(&self, rhs: &Series) -> Self::Item {
+        impl_ineq_compare!(self, rhs, lt, "<")
+    }
+
+    /// Create a boolean mask by checking if self <= rhs.
+    fn lt_eq(&self, rhs: &Series) -> Self::Item {
+        impl_ineq_compare!(self, rhs, lt_eq, "<=")
+    }
+}
+
+impl<Rhs> ChunkCompareEq<Rhs> for Series
 where
     Rhs: NumericNative,
 {
     type Item = PolarsResult<BooleanChunked>;
 
-    fn equal(&self, rhs: Rhs) -> PolarsResult<BooleanChunked> {
+    fn equal(&self, rhs: Rhs) -> Self::Item {
         validate_types(self.dtype(), &DataType::Int8)?;
         let s = self.to_physical_repr();
         Ok(apply_method_physical_numeric!(&s, equal, rhs))
@@ -203,7 +277,7 @@ where
         Ok(apply_method_physical_numeric!(&s, equal_missing, rhs))
     }
 
-    fn not_equal(&self, rhs: Rhs) -> PolarsResult<BooleanChunked> {
+    fn not_equal(&self, rhs: Rhs) -> Self::Item {
         validate_types(self.dtype(), &DataType::Int8)?;
         let s = self.to_physical_repr();
         Ok(apply_method_physical_numeric!(&s, not_equal, rhs))
@@ -214,33 +288,40 @@ where
         let s = self.to_physical_repr();
         Ok(apply_method_physical_numeric!(&s, not_equal_missing, rhs))
     }
+}
 
-    fn gt(&self, rhs: Rhs) -> PolarsResult<BooleanChunked> {
+impl<Rhs> ChunkCompareIneq<Rhs> for Series
+where
+    Rhs: NumericNative,
+{
+    type Item = PolarsResult<BooleanChunked>;
+
+    fn gt(&self, rhs: Rhs) -> Self::Item {
         validate_types(self.dtype(), &DataType::Int8)?;
         let s = self.to_physical_repr();
         Ok(apply_method_physical_numeric!(&s, gt, rhs))
     }
 
-    fn gt_eq(&self, rhs: Rhs) -> PolarsResult<BooleanChunked> {
+    fn gt_eq(&self, rhs: Rhs) -> Self::Item {
         validate_types(self.dtype(), &DataType::Int8)?;
         let s = self.to_physical_repr();
         Ok(apply_method_physical_numeric!(&s, gt_eq, rhs))
     }
 
-    fn lt(&self, rhs: Rhs) -> PolarsResult<BooleanChunked> {
+    fn lt(&self, rhs: Rhs) -> Self::Item {
         validate_types(self.dtype(), &DataType::Int8)?;
         let s = self.to_physical_repr();
         Ok(apply_method_physical_numeric!(&s, lt, rhs))
     }
 
-    fn lt_eq(&self, rhs: Rhs) -> PolarsResult<BooleanChunked> {
+    fn lt_eq(&self, rhs: Rhs) -> Self::Item {
         validate_types(self.dtype(), &DataType::Int8)?;
         let s = self.to_physical_repr();
         Ok(apply_method_physical_numeric!(&s, lt_eq, rhs))
     }
 }
 
-impl ChunkCompare<&str> for Series {
+impl ChunkCompareEq<&str> for Series {
     type Item = PolarsResult<BooleanChunked>;
 
     fn equal(&self, rhs: &str) -> PolarsResult<BooleanChunked> {
@@ -294,8 +375,12 @@ impl ChunkCompare<&str> for Series {
             _ => Ok(replace_non_null(self.name().clone(), self.0.chunks(), true)),
         }
     }
+}
 
-    fn gt(&self, rhs: &str) -> PolarsResult<BooleanChunked> {
+impl ChunkCompareIneq<&str> for Series {
+    type Item = PolarsResult<BooleanChunked>;
+
+    fn gt(&self, rhs: &str) -> Self::Item {
         validate_types(self.dtype(), &DataType::String)?;
         match self.dtype() {
             DataType::String => Ok(self.str().unwrap().gt(rhs)),
