@@ -169,6 +169,19 @@ macro_rules! impl_dyn_series {
                 self.0.agg_list(groups)
             }
 
+            #[cfg(feature = "bitwise")]
+            unsafe fn agg_and(&self, groups: &GroupsProxy) -> Series {
+                self.0.agg_and(groups)
+            }
+            #[cfg(feature = "bitwise")]
+            unsafe fn agg_or(&self, groups: &GroupsProxy) -> Series {
+                self.0.agg_or(groups)
+            }
+            #[cfg(feature = "bitwise")]
+            unsafe fn agg_xor(&self, groups: &GroupsProxy) -> Series {
+                self.0.agg_xor(groups)
+            }
+
             fn subtract(&self, rhs: &Series) -> PolarsResult<Series> {
                 polars_ensure!(
                     self.dtype() == rhs.dtype(),
@@ -239,7 +252,11 @@ macro_rules! impl_dyn_series {
             }
 
             fn get_metadata(&self) -> Option<RwLockReadGuard<dyn MetadataTrait>> {
-                self.metadata_dyn()
+                self.0.metadata_dyn()
+            }
+
+            fn boxed_metadata<'a>(&'a self) -> Option<Box<dyn MetadataTrait + 'a>> {
+                Some(self.0.boxed_metadata_dyn())
             }
 
             fn bitand(&self, other: &Series) -> PolarsResult<Series> {
@@ -451,9 +468,38 @@ macro_rules! impl_dyn_series {
             fn quantile_reduce(
                 &self,
                 quantile: f64,
-                interpol: QuantileInterpolOptions,
+                method: QuantileMethod,
             ) -> PolarsResult<Scalar> {
-                QuantileAggSeries::quantile_reduce(&self.0, quantile, interpol)
+                QuantileAggSeries::quantile_reduce(&self.0, quantile, method)
+            }
+
+            #[cfg(feature = "bitwise")]
+            fn and_reduce(&self) -> PolarsResult<Scalar> {
+                let dt = <$pdt as PolarsDataType>::get_dtype();
+                let av = self.0.and_reduce().map_or(AnyValue::Null, Into::into);
+
+                Ok(Scalar::new(dt, av))
+            }
+
+            #[cfg(feature = "bitwise")]
+            fn or_reduce(&self) -> PolarsResult<Scalar> {
+                let dt = <$pdt as PolarsDataType>::get_dtype();
+                let av = self.0.or_reduce().map_or(AnyValue::Null, Into::into);
+
+                Ok(Scalar::new(dt, av))
+            }
+
+            #[cfg(feature = "bitwise")]
+            fn xor_reduce(&self) -> PolarsResult<Scalar> {
+                let dt = <$pdt as PolarsDataType>::get_dtype();
+                let av = self.0.xor_reduce().map_or(AnyValue::Null, Into::into);
+
+                Ok(Scalar::new(dt, av))
+            }
+
+            #[cfg(feature = "approx_unique")]
+            fn approx_n_unique(&self) -> PolarsResult<IdxSize> {
+                Ok(ChunkApproxNUnique::approx_n_unique(&self.0))
             }
 
             fn clone_inner(&self) -> Arc<dyn SeriesTrait> {
