@@ -1,5 +1,5 @@
 //! Type agnostic columnar data structure.
-pub use crate::prelude::ChunkCompare;
+pub use crate::prelude::ChunkCompareEq;
 use crate::prelude::*;
 use crate::{HEAD_DEFAULT_LENGTH, TAIL_DEFAULT_LENGTH};
 
@@ -23,6 +23,7 @@ use arrow::offset::Offsets;
 pub use from::*;
 pub use iterator::{SeriesIter, SeriesPhysIter};
 use num_traits::NumCast;
+use polars_utils::itertools::Itertools;
 pub use series_trait::{IsSorted, *};
 
 use crate::chunked_array::cast::CastOptions;
@@ -90,10 +91,11 @@ use crate::POOL;
 ///     .all(|(a, b)| a == *b))
 /// ```
 ///
-/// See all the comparison operators in the [CmpOps trait](crate::chunked_array::ops::ChunkCompare)
+/// See all the comparison operators in the [ChunkCompareEq trait](crate::chunked_array::ops::ChunkCompareEq) and
+/// [ChunkCompareIneq trait](crate::chunked_array::ops::ChunkCompareIneq).
 ///
 /// ## Iterators
-/// The Series variants contain differently typed [ChunkedArray's](crate::chunked_array::ChunkedArray).
+/// The Series variants contain differently typed [ChunkedArray](crate::chunked_array::ChunkedArray)s.
 /// These structs can be turned into iterators, making it possible to use any function/ closure you want
 /// on a Series.
 ///
@@ -256,7 +258,7 @@ impl Series {
 
     pub fn into_frame(self) -> DataFrame {
         // SAFETY: A single-column dataframe cannot have length mismatches or duplicate names
-        unsafe { DataFrame::new_no_checks(vec![self.into()]) }
+        unsafe { DataFrame::new_no_checks(self.len(), vec![self.into()]) }
     }
 
     /// Rename series.
@@ -297,11 +299,6 @@ impl Series {
         Self::try_from((name, array))
     }
 
-    #[cfg(feature = "arrow_rs")]
-    pub fn from_arrow_rs(name: PlSmallStr, array: &dyn arrow_array::Array) -> PolarsResult<Series> {
-        Self::from_arrow(name, array.into())
-    }
-
     /// Shrink the capacity of this array to fit its length.
     pub fn shrink_to_fit(&mut self) {
         self._get_inner_mut().shrink_to_fit()
@@ -594,15 +591,17 @@ impl Series {
         lhs.zip_with_same_type(mask, rhs.as_ref())
     }
 
-    /// Cast a datelike Series to their physical representation.
-    /// Primitives remain unchanged
+    /// Converts a Series to their physical representation, if they have one,
+    /// otherwise the series is left unchanged.
     ///
     /// * Date -> Int32
-    /// * Datetime-> Int64
+    /// * Datetime -> Int64
+    /// * Duration -> Int64
     /// * Time -> Int64
     /// * Categorical -> UInt32
     /// * List(inner) -> List(physical of inner)
-    ///
+    /// * Array(inner) -> Array(physical of inner)
+    /// * Struct -> Struct with physical repr of each struct column
     pub fn to_physical_repr(&self) -> Cow<Series> {
         use DataType::*;
         match self.dtype() {
@@ -622,6 +621,11 @@ impl Series {
                 Cow::Owned(ca.physical().clone().into_series())
             },
             List(inner) => Cow::Owned(self.cast(&List(Box::new(inner.to_physical()))).unwrap()),
+            #[cfg(feature = "dtype-array")]
+            Array(inner, size) => Cow::Owned(
+                self.cast(&Array(Box::new(inner.to_physical()), *size))
+                    .unwrap(),
+            ),
             #[cfg(feature = "dtype-struct")]
             Struct(_) => {
                 let arr = self.struct_().unwrap();
@@ -631,7 +635,8 @@ impl Series {
                     .map(|s| s.to_physical_repr().into_owned())
                     .collect();
                 let mut ca =
-                    StructChunked::from_series(self.name().clone(), fields.iter()).unwrap();
+                    StructChunked::from_series(self.name().clone(), arr.len(), fields.iter())
+                        .unwrap();
 
                 if arr.null_count() > 0 {
                     ca.zip_outer_validity(arr);
@@ -642,6 +647,75 @@ impl Series {
         }
     }
 
+    /// Attempts to convert a Series to dtype, only allowing conversions from
+    /// physical to logical dtypes--the inverse of to_physical_repr().
+    ///
+    /// # Safety
+    /// When converting from UInt32 to Categorical it is not checked that the
+    /// values are in-bound for the categorical mapping.
+    pub unsafe fn to_logical_repr_unchecked(&self, dtype: &DataType) -> PolarsResult<Series> {
+        use DataType::*;
+
+        let err = || {
+            Err(
+                polars_err!(ComputeError: "can't cast from {} to {} in to_logical_repr_unchecked", self.dtype(), dtype),
+            )
+        };
+
+        match dtype {
+            dt if self.dtype() == dt => Ok(self.clone()),
+            #[cfg(feature = "dtype-date")]
+            Date => Ok(self.i32()?.clone().into_date().into_series()),
+            #[cfg(feature = "dtype-datetime")]
+            Datetime(u, z) => Ok(self
+                .i64()?
+                .clone()
+                .into_datetime(*u, z.clone())
+                .into_series()),
+            #[cfg(feature = "dtype-duration")]
+            Duration(u) => Ok(self.i64()?.clone().into_duration(*u).into_series()),
+            #[cfg(feature = "dtype-time")]
+            Time => Ok(self.i64()?.clone().into_time().into_series()),
+            #[cfg(feature = "dtype-categorical")]
+            Categorical { .. } | Enum { .. } => {
+                Ok(CategoricalChunked::from_cats_and_dtype_unchecked(
+                    self.u32()?.clone(),
+                    dtype.clone(),
+                )
+                .into_series())
+            },
+            List(inner) => {
+                if let List(self_inner) = self.dtype() {
+                    if inner.to_physical() == **self_inner {
+                        return self.cast(dtype);
+                    }
+                }
+                err()
+            },
+            #[cfg(feature = "dtype-struct")]
+            Struct(target_fields) => {
+                let ca = self.struct_().unwrap();
+                if ca.struct_fields().len() != target_fields.len() {
+                    return err();
+                }
+                let fields = ca
+                    .fields_as_series()
+                    .iter()
+                    .zip(target_fields)
+                    .map(|(s, tf)| s.to_logical_repr_unchecked(tf.dtype()))
+                    .try_collect_vec()?;
+                let mut result =
+                    StructChunked::from_series(self.name().clone(), ca.len(), fields.iter())?;
+                if ca.null_count() > 0 {
+                    result.zip_outer_validity(ca);
+                }
+                Ok(result.into_series())
+            },
+
+            _ => err(),
+        }
+    }
+
     /// Take by index if ChunkedArray contains a single chunk.
     ///
     /// # Safety
@@ -713,10 +787,6 @@ impl Series {
 
     #[cfg(feature = "dtype-time")]
     pub(crate) fn into_time(self) -> Series {
-        #[cfg(not(feature = "dtype-time"))]
-        {
-            panic!("activate feature dtype-time")
-        }
         match self.dtype() {
             DataType::Int64 => self.i64().unwrap().clone().into_time().into_series(),
             DataType::Time => self
@@ -829,6 +899,17 @@ impl Series {
         unsafe { Ok(self.take_unchecked(&idx)) }
     }
 
+    pub fn try_idx(&self) -> Option<&IdxCa> {
+        #[cfg(feature = "bigidx")]
+        {
+            self.try_u64()
+        }
+        #[cfg(not(feature = "bigidx"))]
+        {
+            self.try_u32()
+        }
+    }
+
     pub fn idx(&self) -> PolarsResult<&IdxCa> {
         #[cfg(feature = "bigidx")]
         {
@@ -864,8 +945,7 @@ impl Series {
             DataType::Categorical(Some(rv), _) | DataType::Enum(Some(rv), _) => match &**rv {
                 RevMapping::Local(arr, _) => size += estimated_bytes_size(arr),
                 RevMapping::Global(map, arr, _) => {
-                    size +=
-                        map.capacity() * std::mem::size_of::<u32>() * 2 + estimated_bytes_size(arr);
+                    size += map.capacity() * size_of::<u32>() * 2 + estimated_bytes_size(arr);
                 },
             },
             _ => {},
