@@ -1,4 +1,6 @@
 use arrow::bitmap::Bitmap;
+use arrow::buffer::Buffer;
+use arrow::offset::OffsetsBuffer;
 
 #[cfg(feature = "object")]
 use crate::chunked_array::object::registry::get_object_builder;
@@ -55,7 +57,7 @@ impl Series {
                     .iter()
                     .map(|fld| Series::full_null(fld.name().clone(), size, fld.dtype()))
                     .collect::<Vec<_>>();
-                let ca = StructChunked::from_series(name, fields.iter()).unwrap();
+                let ca = StructChunked::from_series(name, size, fields.iter()).unwrap();
 
                 if !fields.is_empty() {
                     ca.with_outer_validity(Some(Bitmap::new_zeroed(size)))
@@ -64,6 +66,27 @@ impl Series {
                     ca.into_series()
                 }
             },
+            DataType::BinaryOffset => {
+                let length = size as IdxSize;
+
+                let offsets = vec![0; size + 1];
+                let array = BinaryArray::<i64>::new(
+                    dtype.to_arrow(CompatLevel::oldest()),
+                    unsafe { OffsetsBuffer::new_unchecked(Buffer::from(offsets)) },
+                    Buffer::default(),
+                    Some(Bitmap::new_zeroed(size)),
+                );
+
+                unsafe {
+                    BinaryOffsetChunked::new_with_dims(
+                        Arc::new(Field::new(name, dtype.clone())),
+                        vec![Box::new(array)],
+                        length,
+                        length,
+                    )
+                }
+                .into_series()
+            },
             DataType::Null => Series::new_null(name, size),
             DataType::Unknown(kind) => {
                 let dtype = kind.materialize().expect("expected known type");
