@@ -1,14 +1,10 @@
 use std::borrow::Cow;
-#[cfg(feature = "dtype-array")]
-use std::cmp::Ordering;
-#[cfg(feature = "dtype-array")]
-use std::collections::VecDeque;
 
 use arrow::array::*;
+use arrow::bitmap::Bitmap;
 use arrow::legacy::kernels::list::array_to_unit_list;
-use arrow::offset::Offsets;
+use arrow::offset::{Offsets, OffsetsBuffer};
 use polars_error::{polars_bail, polars_ensure, PolarsResult};
-#[cfg(feature = "dtype-array")]
 use polars_utils::format_tuple;
 
 use crate::chunked_array::builder::get_list_builder;
@@ -16,16 +12,11 @@ use crate::datatypes::{DataType, ListChunked};
 use crate::prelude::{IntoSeries, Series, *};
 
 fn reshape_fast_path(name: PlSmallStr, s: &Series) -> Series {
-    let mut ca = match s.dtype() {
-        #[cfg(feature = "dtype-struct")]
-        DataType::Struct(_) => {
-            ListChunked::with_chunk(name, array_to_unit_list(s.array_ref(0).clone()))
-        },
-        _ => ListChunked::from_chunk_iter(
-            name,
-            s.chunks().iter().map(|arr| array_to_unit_list(arr.clone())),
-        ),
-    };
+    let mut ca = ListChunked::from_chunk_iter(
+        name,
+        s.chunks().iter().map(|arr| array_to_unit_list(arr.clone())),
+    );
+
     ca.set_inner_dtype(s.dtype().clone());
     ca.set_fast_explode();
     ca.into_series()
@@ -61,6 +52,35 @@ impl Series {
         }
     }
 
+    /// TODO: Move this somewhere else?
+    pub fn list_offsets_and_validities_recursive(
+        &self,
+    ) -> (Vec<OffsetsBuffer<i64>>, Vec<Option<Bitmap>>) {
+        let mut offsets = vec![];
+        let mut validities = vec![];
+
+        let mut s = self.rechunk();
+
+        while let DataType::List(_) = s.dtype() {
+            let ca = s.list().unwrap();
+            offsets.push(ca.offsets().unwrap());
+            validities.push(ca.rechunk_validity());
+            s = ca.get_inner();
+        }
+
+        (offsets, validities)
+    }
+
+    /// For ListArrays, recursively normalizes the offsets to begin from 0, and
+    /// slices excess length from the values array.
+    pub fn list_rechunk_and_trim_to_normalized_offsets(&self) -> Self {
+        if let Some(ca) = self.try_list() {
+            ca.rechunk_and_trim_to_normalized_offsets().into_series()
+        } else {
+            self.rechunk()
+        }
+    }
+
     /// Convert the values of this Series to a ListChunked with a length of 1,
     /// so a Series of `[1, 2, 3]` becomes `[[1, 2, 3]]`.
     pub fn implode(&self) -> PolarsResult<ListChunked> {
@@ -90,74 +110,101 @@ impl Series {
     }
 
     #[cfg(feature = "dtype-array")]
-    pub fn reshape_array(&self, dimensions: &[i64]) -> PolarsResult<Series> {
+    pub fn reshape_array(&self, dimensions: &[ReshapeDimension]) -> PolarsResult<Series> {
         polars_ensure!(
             !dimensions.is_empty(),
             InvalidOperation: "at least one dimension must be specified"
         );
 
-        let mut dims = dimensions.iter().copied().collect::<VecDeque<_>>();
-
-        let leaf_array = self.get_leaf_array();
+        let leaf_array = self.get_leaf_array().rechunk();
         let size = leaf_array.len();
 
         let mut total_dim_size = 1;
-        let mut infer_dim_index: Option<usize> = None;
-        for (index, &dim) in dims.iter().enumerate() {
-            match dim.cmp(&0) {
-                Ordering::Greater => total_dim_size *= dim as usize,
-                Ordering::Equal => {
-                    polars_ensure!(
-                        index == 0,
-                        InvalidOperation: "cannot reshape array into shape containing a zero dimension after the first: {}",
-                        format_tuple!(dims)
-                    );
-                    total_dim_size = 0;
-                    // We can early exit here, as empty arrays will error with multiple dimensions,
-                    // and non-empty arrays will error when the first dimension is zero.
-                    break;
-                },
-                Ordering::Less => {
-                    polars_ensure!(
-                        infer_dim_index.is_none(),
-                        InvalidOperation: "can only specify one unknown dimension"
-                    );
-                    infer_dim_index = Some(index);
-                },
+        let mut num_infers = 0;
+        for &dim in dimensions {
+            match dim {
+                ReshapeDimension::Infer => num_infers += 1,
+                ReshapeDimension::Specified(dim) => total_dim_size *= dim.get() as usize,
             }
         }
 
+        polars_ensure!(num_infers <= 1, InvalidOperation: "can only specify one inferred dimension");
+
         if size == 0 {
-            if dims.len() > 1 || (infer_dim_index.is_none() && total_dim_size != 0) {
-                polars_bail!(InvalidOperation: "cannot reshape empty array into shape {}", format_tuple!(dims))
-            }
-        } else if total_dim_size == 0 {
-            polars_bail!(InvalidOperation: "cannot reshape non-empty array into shape containing a zero dimension: {}", format_tuple!(dims))
-        } else {
             polars_ensure!(
-                size % total_dim_size == 0,
-                InvalidOperation: "cannot reshape array of size {} into shape {}", size, format_tuple!(dims)
+                num_infers > 0 || total_dim_size == 0,
+                InvalidOperation: "cannot reshape empty array into shape without zero dimension: {}",
+                format_tuple!(dimensions),
             );
+
+            let mut prev_arrow_dtype = leaf_array
+                .dtype()
+                .to_physical()
+                .to_arrow(CompatLevel::newest());
+            let mut prev_dtype = leaf_array.dtype().clone();
+            let mut prev_array = leaf_array.chunks()[0].clone();
+
+            // @NOTE: We need to collect the iterator here because it is lazily processed.
+            let mut current_length = dimensions[0].get_or_infer(0);
+            let len_iter = dimensions[1..]
+                .iter()
+                .map(|d| {
+                    let length = current_length as usize;
+                    current_length *= d.get_or_infer(0);
+                    length
+                })
+                .collect::<Vec<_>>();
+
+            // We pop the outer dimension as that is the height of the series.
+            for (dim, length) in dimensions[1..].iter().zip(len_iter).rev() {
+                // Infer dimension if needed
+                let dim = dim.get_or_infer(0);
+                prev_arrow_dtype = prev_arrow_dtype.to_fixed_size_list(dim as usize, true);
+                prev_dtype = DataType::Array(Box::new(prev_dtype), dim as usize);
+
+                prev_array =
+                    FixedSizeListArray::new(prev_arrow_dtype.clone(), length, prev_array, None)
+                        .boxed();
+            }
+
+            return Ok(unsafe {
+                Series::from_chunks_and_dtype_unchecked(
+                    leaf_array.name().clone(),
+                    vec![prev_array],
+                    &prev_dtype,
+                )
+            });
         }
 
-        // Infer dimension
-        if let Some(index) = infer_dim_index {
-            let inferred_dim = size / total_dim_size;
-            let item = dims.get_mut(index).unwrap();
-            *item = i64::try_from(inferred_dim).unwrap();
-        }
+        polars_ensure!(
+            total_dim_size > 0,
+            InvalidOperation: "cannot reshape non-empty array into shape containing a zero dimension: {}",
+            format_tuple!(dimensions)
+        );
+
+        polars_ensure!(
+            size % total_dim_size == 0,
+            InvalidOperation: "cannot reshape array of size {} into shape {}", size, format_tuple!(dimensions)
+        );
 
         let leaf_array = leaf_array.rechunk();
+        let mut prev_arrow_dtype = leaf_array
+            .dtype()
+            .to_physical()
+            .to_arrow(CompatLevel::newest());
         let mut prev_dtype = leaf_array.dtype().clone();
         let mut prev_array = leaf_array.chunks()[0].clone();
 
         // We pop the outer dimension as that is the height of the series.
-        let _ = dims.pop_front();
-        while let Some(dim) = dims.pop_back() {
+        for dim in dimensions[1..].iter().rev() {
+            // Infer dimension if needed
+            let dim = dim.get_or_infer((size / total_dim_size) as u64);
+            prev_arrow_dtype = prev_arrow_dtype.to_fixed_size_list(dim as usize, true);
             prev_dtype = DataType::Array(Box::new(prev_dtype), dim as usize);
 
             prev_array = FixedSizeListArray::new(
-                prev_dtype.to_arrow(CompatLevel::newest()),
+                prev_arrow_dtype.clone(),
+                prev_array.len() / dim as usize,
                 prev_array,
                 None,
             )
@@ -172,7 +219,7 @@ impl Series {
         })
     }
 
-    pub fn reshape_list(&self, dimensions: &[i64]) -> PolarsResult<Series> {
+    pub fn reshape_list(&self, dimensions: &[ReshapeDimension]) -> PolarsResult<Series> {
         polars_ensure!(
             !dimensions.is_empty(),
             InvalidOperation: "at least one dimension must be specified"
@@ -187,38 +234,43 @@ impl Series {
 
         let s_ref = s.as_ref();
 
-        let dimensions = dimensions.to_vec();
+        // let dimensions = dimensions.to_vec();
 
         match dimensions.len() {
             1 => {
                 polars_ensure!(
-                    dimensions[0] as usize == s_ref.len() || dimensions[0] == -1_i64,
+                    dimensions[0].get().map_or(true, |dim| dim as usize == s_ref.len()),
                     InvalidOperation: "cannot reshape len {} into shape {:?}", s_ref.len(), dimensions,
                 );
                 Ok(s_ref.clone())
             },
             2 => {
-                let mut rows = dimensions[0];
-                let mut cols = dimensions[1];
+                let rows = dimensions[0];
+                let cols = dimensions[1];
 
                 if s_ref.len() == 0_usize {
-                    if (rows == -1 || rows == 0) && (cols == -1 || cols == 0 || cols == 1) {
+                    if rows.get_or_infer(0) == 0 && cols.get_or_infer(0) <= 1 {
                         let s = reshape_fast_path(s.name().clone(), s_ref);
                         return Ok(s);
                     } else {
-                        polars_bail!(InvalidOperation: "cannot reshape len 0 into shape {:?}", dimensions,)
+                        polars_bail!(InvalidOperation: "cannot reshape len 0 into shape {}", format_tuple!(dimensions))
                     }
                 }
 
+                use ReshapeDimension as RD;
                 // Infer dimension.
-                if rows == -1 && cols >= 1 {
-                    rows = s_ref.len() as i64 / cols
-                } else if cols == -1 && rows >= 1 {
-                    cols = s_ref.len() as i64 / rows
-                } else if rows == -1 && cols == -1 {
-                    rows = s_ref.len() as i64;
-                    cols = 1_i64;
-                }
+
+                let (rows, cols) = match (rows, cols) {
+                    (RD::Infer, RD::Specified(cols)) if cols.get() >= 1 => {
+                        (s_ref.len() as u64 / cols.get(), cols.get())
+                    },
+                    (RD::Specified(rows), RD::Infer) if rows.get() >= 1 => {
+                        (rows.get(), s_ref.len() as u64 / rows.get())
+                    },
+                    (RD::Infer, RD::Infer) => (s_ref.len() as u64, 1u64),
+                    (RD::Specified(rows), RD::Specified(cols)) => (rows.get(), cols.get()),
+                    _ => polars_bail!(InvalidOperation: "reshape of non-zero list into zero list"),
+                };
 
                 // Fast path, we can create a unit list so we only allocate offsets.
                 if rows as usize == s_ref.len() && cols == 1 {
@@ -232,11 +284,11 @@ impl Series {
                 );
 
                 let mut builder =
-                    get_list_builder(s_ref.dtype(), s_ref.len(), rows as usize, s.name().clone())?;
+                    get_list_builder(s_ref.dtype(), s_ref.len(), rows as usize, s.name().clone());
 
-                let mut offset = 0i64;
+                let mut offset = 0u64;
                 for _ in 0..rows {
-                    let row = s_ref.slice(offset, cols as usize);
+                    let row = s_ref.slice(offset as i64, cols as usize);
                     builder.append_series(&row).unwrap();
                     offset += cols;
                 }
@@ -258,7 +310,7 @@ mod test {
     fn test_to_list() -> PolarsResult<()> {
         let s = Series::new("a".into(), &[1, 2, 3]);
 
-        let mut builder = get_list_builder(s.dtype(), s.len(), 1, s.name().clone())?;
+        let mut builder = get_list_builder(s.dtype(), s.len(), 1, s.name().clone());
         builder.append_series(&s).unwrap();
         let expected = builder.finish();
 
@@ -279,7 +331,11 @@ mod test {
             (&[-1, 2], 2),
             (&[2, -1], 2),
         ] {
-            let out = s.reshape_list(dims)?;
+            let dims = dims
+                .iter()
+                .map(|&v| ReshapeDimension::new(v))
+                .collect::<Vec<_>>();
+            let out = s.reshape_list(&dims)?;
             assert_eq!(out.len(), list_len);
             assert!(matches!(out.dtype(), DataType::List(_)));
             assert_eq!(out.explode()?.len(), 4);
