@@ -498,3 +498,56 @@ fn materialize_smallest_dyn_int(v: i128) -> AnyValue<'static> {
         },
     }
 }
+
+pub fn merge_dtypes_many<I: IntoIterator<Item = D> + Clone, D: AsRef<DataType>>(
+    into_iter: I,
+) -> PolarsResult<DataType> {
+    let mut iter = into_iter.clone().into_iter();
+
+    let mut st = iter
+        .next()
+        .ok_or_else(|| polars_err!(ComputeError: "expect at least 1 dtype"))
+        .map(|d| d.as_ref().clone())?;
+
+    for d in iter {
+        st = try_get_supertype(d.as_ref(), &st)?;
+    }
+
+    match st {
+        #[cfg(feature = "dtype-categorical")]
+        DataType::Categorical(Some(_), ordering) => {
+            // This merges the global rev maps with linear complexity.
+            // If we do a binary reduce, it would be quadratic.
+            let mut iter = into_iter.into_iter();
+            let first_dt = iter.next().unwrap();
+            let first_dt = first_dt.as_ref();
+            let DataType::Categorical(Some(rm), _) = first_dt else {
+                unreachable!()
+            };
+
+            let mut merger = GlobalRevMapMerger::new(rm.clone());
+
+            for d in iter {
+                if let DataType::Categorical(Some(rm), _) = d.as_ref() {
+                    merger.merge_map(rm)?
+                }
+            }
+            let rev_map = merger.finish();
+
+            Ok(DataType::Categorical(Some(rev_map), ordering))
+        },
+        // This would be quadratic if we do this with the binary `merge_dtypes`.
+        DataType::List(inner) if inner.contains_categoricals() => {
+            polars_bail!(ComputeError: "merging nested categoricals not yet supported")
+        },
+        #[cfg(feature = "dtype-array")]
+        DataType::Array(inner, _) if inner.contains_categoricals() => {
+            polars_bail!(ComputeError: "merging nested categoricals not yet supported")
+        },
+        #[cfg(feature = "dtype-struct")]
+        DataType::Struct(fields) if fields.iter().any(|f| f.dtype().contains_categoricals()) => {
+            polars_bail!(ComputeError: "merging nested categoricals not yet supported")
+        },
+        _ => Ok(st),
+    }
+}
