@@ -175,6 +175,24 @@ impl PhysicalExpr for AggregationExpr {
                 .var_reduce(ddof)
                 .map(|sc| sc.into_series(s.name().clone())),
             GroupByMethod::Quantile(_, _) => unimplemented!(),
+            #[cfg(feature = "bitwise")]
+            GroupByMethod::Bitwise(f) => match f {
+                GroupByBitwiseMethod::And => parallel_op_series(
+                    |s| s.and_reduce().map(|sc| sc.into_series(s.name().clone())),
+                    s,
+                    allow_threading,
+                ),
+                GroupByBitwiseMethod::Or => parallel_op_series(
+                    |s| s.or_reduce().map(|sc| sc.into_series(s.name().clone())),
+                    s,
+                    allow_threading,
+                ),
+                GroupByBitwiseMethod::Xor => parallel_op_series(
+                    |s| s.xor_reduce().map(|sc| sc.into_series(s.name().clone())),
+                    s,
+                    allow_threading,
+                ),
+            },
         }
     }
     #[allow(clippy::ptr_arg)]
@@ -375,7 +393,12 @@ impl PhysicalExpr for AggregationExpr {
                     let s = match ac.agg_state() {
                         // mean agg:
                         // -> f64 -> list<f64>
-                        AggState::AggregatedScalar(s) => s.reshape_list(&[-1, 1]).unwrap(),
+                        AggState::AggregatedScalar(s) => s
+                            .reshape_list(&[
+                                ReshapeDimension::Infer,
+                                ReshapeDimension::new_dimension(1),
+                            ])
+                            .unwrap(),
                         _ => {
                             let agg = ac.aggregated();
                             agg.as_list().into_series()
@@ -402,6 +425,16 @@ impl PhysicalExpr for AggregationExpr {
                     // implemented explicitly in AggQuantile struct
                     unimplemented!()
                 },
+                #[cfg(feature = "bitwise")]
+                GroupByMethod::Bitwise(f) => {
+                    let (s, groups) = ac.get_final_aggregation();
+                    let agg_s = match f {
+                        GroupByBitwiseMethod::And => s.agg_and(&groups),
+                        GroupByBitwiseMethod::Or => s.agg_or(&groups),
+                        GroupByBitwiseMethod::Xor => s.agg_xor(&groups),
+                    };
+                    AggregatedScalar(rename_series(agg_s, keep_name))
+                },
                 GroupByMethod::NanMin => {
                     #[cfg(feature = "propagate_nans")]
                     {
@@ -502,11 +535,13 @@ impl PartitionedAggregation for AggregationExpr {
                         };
                         let mut count_s = series.agg_valid_count(groups);
                         count_s.rename(PlSmallStr::from_static("__POLARS_COUNT"));
-                        Ok(
-                            StructChunked::from_series(new_name, [agg_s, count_s].iter())
-                                .unwrap()
-                                .into_series(),
+                        Ok(StructChunked::from_series(
+                            new_name,
+                            agg_s.len(),
+                            [agg_s, count_s].iter(),
                         )
+                        .unwrap()
+                        .into_series())
                     }
                 },
                 GroupByMethod::Implode => {
@@ -680,19 +715,19 @@ impl PartitionedAggregation for AggregationExpr {
 pub struct AggQuantileExpr {
     pub(crate) input: Arc<dyn PhysicalExpr>,
     pub(crate) quantile: Arc<dyn PhysicalExpr>,
-    pub(crate) interpol: QuantileInterpolOptions,
+    pub(crate) method: QuantileMethod,
 }
 
 impl AggQuantileExpr {
     pub fn new(
         input: Arc<dyn PhysicalExpr>,
         quantile: Arc<dyn PhysicalExpr>,
-        interpol: QuantileInterpolOptions,
+        method: QuantileMethod,
     ) -> Self {
         Self {
             input,
             quantile,
-            interpol,
+            method,
         }
     }
 
@@ -715,7 +750,7 @@ impl PhysicalExpr for AggQuantileExpr {
         let input = self.input.evaluate(df, state)?;
         let quantile = self.get_quantile(df, state)?;
         input
-            .quantile_reduce(quantile, self.interpol)
+            .quantile_reduce(quantile, self.method)
             .map(|sc| sc.into_series(input.name().clone()))
     }
     #[allow(clippy::ptr_arg)]
@@ -736,7 +771,7 @@ impl PhysicalExpr for AggQuantileExpr {
         let mut agg = unsafe {
             ac.flat_naive()
                 .into_owned()
-                .agg_quantile(ac.groups(), quantile, self.interpol)
+                .agg_quantile(ac.groups(), quantile, self.method)
         };
         agg.rename(keep_name);
         Ok(AggregationContext::from_agg_state(
