@@ -55,12 +55,12 @@ fn apply_operator_owned(left: Series, right: Series, op: Operator) -> PolarsResu
 pub fn apply_operator(left: &Series, right: &Series, op: Operator) -> PolarsResult<Series> {
     use DataType::*;
     match op {
-        Operator::Gt => ChunkCompare::gt(left, right).map(|ca| ca.into_series()),
-        Operator::GtEq => ChunkCompare::gt_eq(left, right).map(|ca| ca.into_series()),
-        Operator::Lt => ChunkCompare::lt(left, right).map(|ca| ca.into_series()),
-        Operator::LtEq => ChunkCompare::lt_eq(left, right).map(|ca| ca.into_series()),
-        Operator::Eq => ChunkCompare::equal(left, right).map(|ca| ca.into_series()),
-        Operator::NotEq => ChunkCompare::not_equal(left, right).map(|ca| ca.into_series()),
+        Operator::Gt => ChunkCompareIneq::gt(left, right).map(|ca| ca.into_series()),
+        Operator::GtEq => ChunkCompareIneq::gt_eq(left, right).map(|ca| ca.into_series()),
+        Operator::Lt => ChunkCompareIneq::lt(left, right).map(|ca| ca.into_series()),
+        Operator::LtEq => ChunkCompareIneq::lt_eq(left, right).map(|ca| ca.into_series()),
+        Operator::Eq => ChunkCompareEq::equal(left, right).map(|ca| ca.into_series()),
+        Operator::NotEq => ChunkCompareEq::not_equal(left, right).map(|ca| ca.into_series()),
         Operator::Plus => left + right,
         Operator::Minus => left - right,
         Operator::Multiply => left * right,
@@ -75,11 +75,8 @@ pub fn apply_operator(left: &Series, right: &Series, op: Operator) -> PolarsResu
                 let right_dt = right.dtype().cast_leaf(Float64);
                 left.cast(&left_dt)? / right.cast(&right_dt)?
             },
-            dt @ List(_) => {
-                let left_dt = dt.cast_leaf(Float64);
-                let right_dt = right.dtype().cast_leaf(Float64);
-                left.cast(&left_dt)? / right.cast(&right_dt)?
-            },
+            List(_) => left / right,
+            _ if right.dtype().is_list() => left / right,
             _ => {
                 if right.dtype().is_temporal() {
                     return left / right;
@@ -283,7 +280,7 @@ mod stats {
     use super::*;
 
     fn apply_operator_stats_eq(min_max: &Series, literal: &Series) -> bool {
-        use ChunkCompare as C;
+        use ChunkCompareIneq as C;
         // Literal is greater than max, don't need to read.
         if C::gt(literal, min_max).map(|s| s.all()).unwrap_or(false) {
             return false;
@@ -301,7 +298,7 @@ mod stats {
         if min_max.len() < 2 || min_max.null_count() > 0 {
             return true;
         }
-        use ChunkCompare as C;
+        use ChunkCompareEq as C;
 
         // First check proofs all values are the same (e.g. min/max is the same)
         // Second check proofs all values are equal, so we can skip as we search
@@ -315,7 +312,7 @@ mod stats {
     }
 
     fn apply_operator_stats_rhs_lit(min_max: &Series, literal: &Series, op: Operator) -> bool {
-        use ChunkCompare as C;
+        use ChunkCompareIneq as C;
         match op {
             Operator::Eq => apply_operator_stats_eq(min_max, literal),
             Operator::NotEq => apply_operator_stats_neq(min_max, literal),
@@ -351,10 +348,10 @@ mod stats {
     }
 
     fn apply_operator_stats_lhs_lit(literal: &Series, min_max: &Series, op: Operator) -> bool {
-        use ChunkCompare as C;
+        use ChunkCompareIneq as C;
         match op {
             Operator::Eq => apply_operator_stats_eq(min_max, literal),
-            Operator::NotEq => apply_operator_stats_eq(min_max, literal),
+            Operator::NotEq => apply_operator_stats_neq(min_max, literal),
             Operator::Gt => {
                 // Literal is bigger than max value, selection needs all rows.
                 C::gt(literal, min_max).map(|ca| ca.any()).unwrap_or(false)
@@ -457,10 +454,6 @@ mod stats {
 
     impl StatsEvaluator for BinaryExpr {
         fn should_read(&self, stats: &BatchStats) -> PolarsResult<bool> {
-            if std::env::var("POLARS_NO_PARQUET_STATISTICS").is_ok() {
-                return Ok(true);
-            }
-
             use Operator::*;
             match (
                 self.left.as_stats_evaluator(),
