@@ -72,13 +72,6 @@ impl AggState {
             AggState::NotAggregated(s) => AggState::NotAggregated(func(s)?),
         })
     }
-
-    fn map<F>(&self, func: F) -> Self
-    where
-        F: FnOnce(&Series) -> Series,
-    {
-        self.try_map(|s| Ok(func(s))).unwrap()
-    }
 }
 
 // lazy update strategy
@@ -421,7 +414,12 @@ impl<'a> AggregationContext<'a> {
                 self.groups();
                 let rows = self.groups.len();
                 let s = s.new_from_index(0, rows);
-                let out = s.reshape_list(&[rows as i64, -1]).unwrap();
+                let out = s
+                    .reshape_list(&[
+                        ReshapeDimension::new_dimension(rows as u64),
+                        ReshapeDimension::Infer,
+                    ])
+                    .unwrap();
                 self.state = AggState::AggregatedList(out.clone());
                 out
             },
