@@ -12,6 +12,7 @@ pub struct TernaryExpr {
     expr: Expr,
     // Can be expensive on small data to run literals in parallel.
     run_par: bool,
+    returns_scalar: bool,
 }
 
 impl TernaryExpr {
@@ -21,6 +22,7 @@ impl TernaryExpr {
         falsy: Arc<dyn PhysicalExpr>,
         expr: Expr,
         run_par: bool,
+        returns_scalar: bool,
     ) -> Self {
         Self {
             predicate,
@@ -28,6 +30,7 @@ impl TernaryExpr {
             falsy,
             expr,
             run_par,
+            returns_scalar,
         }
     }
 }
@@ -53,7 +56,7 @@ fn finish_as_iters<'a>(
             .transpose()
         })
         .collect::<PolarsResult<ListChunked>>()?
-        .with_name(ac_truthy.series().name());
+        .with_name(ac_truthy.series().name().clone());
 
     // Aggregation leaves only a single chunk.
     let arr = ca.downcast_iter().next().unwrap();
@@ -280,12 +283,12 @@ impl PhysicalExpr for TernaryExpr {
                 let values = out.array_ref(0);
                 let offsets = ac_target.series().list().unwrap().offsets()?;
                 let inner_type = out.dtype();
-                let data_type = LargeListArray::default_datatype(values.data_type().clone());
+                let dtype = LargeListArray::default_datatype(values.dtype().clone());
 
                 // SAFETY: offsets are correct.
-                let out = LargeListArray::new(data_type, offsets, values.clone(), None);
+                let out = LargeListArray::new(dtype, offsets, values.clone(), None);
 
-                let mut out = ListChunked::with_chunk(truthy.name(), out);
+                let mut out = ListChunked::with_chunk(truthy.name().clone(), out);
                 unsafe { out.to_logical(inner_type.clone()) };
 
                 if ac_target.series().list().unwrap()._can_fast_explode() {
@@ -322,6 +325,10 @@ impl PhysicalExpr for TernaryExpr {
     fn as_partitioned_aggregator(&self) -> Option<&dyn PartitionedAggregation> {
         Some(self)
     }
+
+    fn is_scalar(&self) -> bool {
+        self.returns_scalar
+    }
 }
 
 impl PartitionedAggregation for TernaryExpr {
