@@ -1,4 +1,4 @@
-use polars_core::prelude::CompatLevel;
+use polars_core::prelude::{Column, CompatLevel};
 
 use super::*;
 
@@ -53,8 +53,16 @@ unsafe extern "C" fn c_release_series_export(e: *mut SeriesExport) {
     e.release = None;
 }
 
+pub fn export_column(c: &Column) -> SeriesExport {
+    export_series(c.as_materialized_series())
+}
+
 pub fn export_series(s: &Series) -> SeriesExport {
-    let field = ArrowField::new(s.name(), s.dtype().to_arrow(CompatLevel::newest()), true);
+    let field = ArrowField::new(
+        s.name().clone(),
+        s.dtype().to_arrow(CompatLevel::newest()),
+        true,
+    );
     let schema = Box::new(ffi::export_field_to_c(&field));
 
     let mut arrays = (0..s.chunks().len())
@@ -91,7 +99,7 @@ pub unsafe fn import_series(e: SeriesExport) -> PolarsResult<Series> {
         })
         .collect::<PolarsResult<Vec<_>>>()?;
 
-    Series::try_from((field.name.as_str(), chunks))
+    Series::try_from((field.name.clone(), chunks))
 }
 
 /// # Safety
@@ -144,7 +152,7 @@ mod test {
 
     #[test]
     fn test_ffi() {
-        let s = Series::new("a", [1, 2]);
+        let s = Series::new("a".into(), [1, 2]);
         let e = export_series(&s);
 
         unsafe {
