@@ -136,7 +136,7 @@ fn test_parquet_statistics() -> PolarsResult<()> {
 
     // issue: 13427
     let out = scan_foods_parquet(par)
-        .filter(col("calories").is_in(lit(Series::new("", [0, 500]))))
+        .filter(col("calories").is_in(lit(Series::new("".into(), [0, 500]))))
         .collect()?;
     assert_eq!(out.shape(), (0, 4));
 
@@ -417,7 +417,6 @@ fn test_ipc_globbing() -> PolarsResult<()> {
             cache: true,
             rechunk: false,
             row_index: None,
-            memory_map: true,
             cloud_options: None,
             hive_options: Default::default(),
             include_file_paths: None,
@@ -590,7 +589,7 @@ fn test_row_index_on_files() -> PolarsResult<()> {
     for offset in [0 as IdxSize, 10] {
         let lf = LazyCsvReader::new(FOODS_CSV)
             .with_row_index(Some(RowIndex {
-                name: Arc::from("index"),
+                name: PlSmallStr::from_static("index"),
                 offset,
             }))
             .finish()?;
@@ -665,7 +664,7 @@ fn scan_anonymous_fn_with_options() -> PolarsResult<()> {
         fn scan(&self, scan_opts: AnonymousScanArgs) -> PolarsResult<DataFrame> {
             assert_eq!(scan_opts.with_columns.clone().unwrap().len(), 2);
             assert_eq!(scan_opts.n_rows, Some(3));
-            let out = fruits_cars().select(scan_opts.with_columns.unwrap().as_ref())?;
+            let out = fruits_cars().select(scan_opts.with_columns.unwrap().iter().cloned())?;
             Ok(out.slice(0, scan_opts.n_rows.unwrap()))
         }
     }
@@ -701,7 +700,7 @@ fn scan_small_dtypes() -> PolarsResult<()> {
         let df = LazyCsvReader::new(FOODS_CSV)
             .with_has_header(true)
             .with_dtype_overwrite(Some(Arc::new(Schema::from_iter([Field::new(
-                "sugars_g",
+                "sugars_g".into(),
                 dt.clone(),
             )]))))
             .finish()?
