@@ -21,7 +21,7 @@ pub(crate) use ndjson::JsonExec;
 pub(crate) use parquet::ParquetExec;
 #[cfg(any(feature = "ipc", feature = "parquet", feature = "csv"))]
 use polars_io::predicates::PhysicalIoExpr;
-#[cfg(any(feature = "parquet", feature = "csv", feature = "ipc", feature = "cse"))]
+#[cfg(any(feature = "parquet", feature = "csv", feature = "ipc"))]
 use polars_io::prelude::*;
 use polars_plan::global::_set_n_rows_for_scan;
 
@@ -30,39 +30,11 @@ pub(crate) use self::python_scan::*;
 use super::*;
 use crate::prelude::*;
 
-#[cfg(any(feature = "ipc", feature = "parquet"))]
-type Projection = Option<Vec<usize>>;
-#[cfg(any(feature = "ipc", feature = "parquet"))]
-type Predicate = Option<Arc<dyn PhysicalIoExpr>>;
-
-#[cfg(any(feature = "ipc", feature = "parquet"))]
-fn prepare_scan_args(
-    predicate: Option<Arc<dyn PhysicalExpr>>,
-    with_columns: &mut Option<Arc<[String]>>,
-    schema: &mut SchemaRef,
-    has_row_index: bool,
-    hive_partitions: Option<&[Series]>,
-) -> (Projection, Predicate) {
-    let with_columns = mem::take(with_columns);
-    let schema = mem::take(schema);
-
-    let projection = materialize_projection(
-        with_columns.as_deref(),
-        &schema,
-        hive_partitions,
-        has_row_index,
-    );
-
-    let predicate = predicate.map(phys_expr_to_io_expr);
-
-    (projection, predicate)
-}
-
 /// Producer of an in memory DataFrame
 pub struct DataFrameExec {
     pub(crate) df: Arc<DataFrame>,
     pub(crate) filter: Option<Arc<dyn PhysicalExpr>>,
-    pub(crate) projection: Option<Vec<SmartString>>,
+    pub(crate) projection: Option<Vec<PlSmallStr>>,
     pub(crate) predicate_has_windows: bool,
 }
 
@@ -74,7 +46,7 @@ impl Executor for DataFrameExec {
         // projection should be before selection as those are free
         // TODO: this is only the case if we don't create new columns
         if let Some(projection) = &self.projection {
-            df = df.select(projection.as_slice())?;
+            df = df.select(projection.iter().cloned())?;
         }
 
         if let Some(selection) = &self.filter {
