@@ -64,7 +64,7 @@ impl StackExec {
                 // new, unique column names. It is immediately
                 // followed by a projection which pulls out the
                 // possibly mismatching column lengths.
-                unsafe { df.get_columns_mut() }.extend(res.into_iter().map(Column::from));
+                unsafe { df.column_extend_unchecked(res.into_iter().map(Column::from)) };
             } else {
                 let (df_height, df_width) = df.shape();
 
@@ -79,11 +79,19 @@ impl StackExec {
                 for (i, c) in res.iter().enumerate() {
                     let len = c.len();
                     if verify_scalar && len != df_height && len == 1 && df_width > 0 {
-                        polars_ensure!(self.exprs[i].is_scalar(),
-                            InvalidOperation: "Series {}, length {} doesn't match the DataFrame height of {}\n\n\
-                            If you want this Series to be broadcasted, ensure it is a scalar (for instance by adding '.first()').",
-                            c.name(), len, df_height
-                        );
+                        #[allow(clippy::collapsible_if)]
+                        if !self.exprs[i].is_scalar()
+                            && std::env::var("POLARS_ALLOW_NON_SCALAR_EXP").as_deref() != Ok("1")
+                        {
+                            let identifier = match self.exprs[i].as_expression() {
+                                Some(e) => format!("expression: {}", e),
+                                None => "this Series".to_string(),
+                            };
+                            polars_bail!(InvalidOperation: "Series {}, length {} doesn't match the DataFrame height of {}\n\n\
+                                If you want {} to be broadcasted, ensure it is a scalar (for instance by adding '.first()').",
+                                c.name(), len, df_height, identifier
+                            );
+                        }
                     }
                 }
                 df._add_series(res, schema)?;
