@@ -233,9 +233,9 @@ fn pivot_impl(
             already exists in the DataFrame. Please rename it prior to calling `pivot`.")
         }
         // @scalar-opt
-        let columns_struct = StructChunked::from_columns(column.clone(), fields)
+        let columns_struct = StructChunked::from_columns(column.clone(), fields[0].len(), fields)
             .unwrap()
-            .into_series();
+            .into_column();
         let mut binding = pivot_df.clone();
         let pivot_df = unsafe { binding.with_column_unchecked(columns_struct) };
         pivot_impl_single_column(
