@@ -35,7 +35,7 @@ pub fn impl_duration(s: &[Column], time_unit: TimeUnit) -> PolarsResult<Column>
                 microseconds = (microseconds + (nanoseconds.wrapping_trunc_div_scalar(1_000)))?;
             }
             if !is_zero_scalar(&milliseconds) {
-                microseconds = (microseconds + (milliseconds * 1_000)?)?;
+                microseconds = (microseconds + milliseconds * 1_000)?;
             }
             microseconds
         },
@@ -44,10 +44,10 @@ pub fn impl_duration(s: &[Column], time_unit: TimeUnit) -> PolarsResult<Column>
                 nanoseconds = nanoseconds.new_from_index(0, max_len);
             }
             if !is_zero_scalar(&microseconds) {
-                nanoseconds = (nanoseconds + (microseconds * 1_000)?)?;
+                nanoseconds = (nanoseconds + microseconds * 1_000)?;
             }
             if !is_zero_scalar(&milliseconds) {
-                nanoseconds = (nanoseconds + (milliseconds * 1_000_000)?)?;
+                nanoseconds = (nanoseconds + milliseconds * 1_000_000)?;
             }
             nanoseconds
         },
@@ -72,19 +72,19 @@ pub fn impl_duration(s: &[Column], time_unit: TimeUnit) -> PolarsResult<Column>
         TimeUnit::Milliseconds => MILLISECONDS,
     };
     if !is_zero_scalar(&seconds) {
-        duration = ((duration + seconds)? * multiplier)?;
+        duration = (duration + seconds * multiplier)?;
     }
     if !is_zero_scalar(&minutes) {
-        duration = ((duration + minutes)? * (multiplier * 60))?;
+        duration = (duration + minutes * multiplier * 60)?;
     }
     if !is_zero_scalar(&hours) {
-        duration = ((duration + hours)? * (multiplier * 60 * 60))?;
+        duration = (duration + hours * multiplier * 60 * 60)?;
     }
     if !is_zero_scalar(&days) {
-        duration = ((duration + days)? * (multiplier * SECONDS_IN_DAY))?;
+        duration = (duration + days * multiplier * SECONDS_IN_DAY)?;
     }
     if !is_zero_scalar(&weeks) {
-        duration = ((duration + weeks)? * (multiplier * SECONDS_IN_DAY * 7))?;
+        duration = (duration + weeks * multiplier * SECONDS_IN_DAY * 7)?;
     }
 
     duration
