@@ -3,7 +3,7 @@ use std::collections::VecDeque;
 
 use arrow::array::*;
 use arrow::datatypes::{ArrowDataType, Field, IntervalUnit, PhysicalType};
-use arrow::types::i256;
+use arrow::types::{f16, i256, NativeType};
 use arrow::with_match_primitive_type_full;
 use ethnum::I256;
 use polars_error::{polars_bail, PolarsResult};
@@ -28,6 +28,7 @@ mod struct_;
 mod utf8;
 
 use self::list::DynMutableListArray;
+use super::PrimitiveLogicalType;
 
 /// Arrow-deserialized parquet Statistics of a file
 #[derive(Debug, PartialEq)]
@@ -319,7 +320,11 @@ fn push(
                 null_count,
             );
         },
-        Struct(_) => {
+        Struct(fields) => {
+            if fields.is_empty() {
+                return Ok(());
+            }
+
             let min = min
                 .as_mut_any()
                 .downcast_mut::<struct_::DynMutableStructArray>()
@@ -338,11 +343,11 @@ fn push(
                 .unwrap();
 
             return min
-                .inner
+                .inner_mut()
                 .iter_mut()
-                .zip(max.inner.iter_mut())
-                .zip(distinct_count.inner.iter_mut())
-                .zip(null_count.inner.iter_mut())
+                .zip(max.inner_mut())
+                .zip(distinct_count.inner_mut())
+                .zip(null_count.inner_mut())
                 .try_for_each(|(((min, max), distinct_count), null_count)| {
                     push(
                         stats,
@@ -545,6 +550,37 @@ fn push(
     }
 }
 
+pub fn cast_statistics(
+    statistics: ParquetStatistics,
+    primitive_type: &ParquetPrimitiveType,
+    output_type: &ArrowDataType,
+) -> ParquetStatistics {
+    use {ArrowDataType as DT, PrimitiveLogicalType as PT};
+
+    match (primitive_type.logical_type, output_type) {
+        (Some(PT::Float16), DT::Float32) => {
+            let statistics = statistics.expect_fixedlen();
+
+            let primitive_type = primitive_type.clone();
+
+            ParquetStatistics::Float(PrimitiveStatistics::<f32> {
+                primitive_type,
+                null_count: statistics.null_count,
+                distinct_count: statistics.distinct_count,
+                min_value: statistics
+                    .min_value
+                    .as_ref()
+                    .map(|v| f16::from_le_bytes([v[0], v[1]]).to_f32()),
+                max_value: statistics
+                    .max_value
+                    .as_ref()
+                    .map(|v| f16::from_le_bytes([v[0], v[1]]).to_f32()),
+            })
+        },
+        _ => statistics,
+    }
+}
+
 /// Deserializes the statistics in the column chunks from a single `row_group`
 /// into [`Statistics`] associated from `field`'s name.
 ///
@@ -558,9 +594,13 @@ pub fn deserialize<'a>(
 
     let mut stats = field_md
         .map(|column| {
+            let primitive_type = &column.descriptor().descriptor.primitive_type;
             Ok((
-                column.statistics().transpose()?,
-                column.descriptor().descriptor.primitive_type.clone(),
+                column
+                    .statistics()
+                    .transpose()?
+                    .map(|stats| cast_statistics(stats, primitive_type, &field.dtype)),
+                primitive_type.clone(),
             ))
         })
         .collect::<PolarsResult<VecDeque<(Option<ParquetStatistics>, ParquetPrimitiveType)>>>()?;
