@@ -279,7 +279,7 @@ impl<const FIXED: bool> AggHashTable<FIXED> {
                 .map(|buf| buf.into_series().into_column()),
         );
         physical_agg_to_logical(&mut cols, &self.output_schema);
-        unsafe { DataFrame::new_no_checks(cols) }
+        unsafe { DataFrame::new_no_checks_height_from_first(cols) }
     }
 }
 
