@@ -158,29 +158,30 @@ impl ArrayNameSpace {
     }
 
     #[cfg(feature = "array_to_struct")]
-    pub fn to_struct(self, name_generator: Option<ArrToStructNameGenerator>) -> Expr {
-        self.0
+    pub fn to_struct(self, name_generator: Option<ArrToStructNameGenerator>) -> PolarsResult<Expr> {
+        Ok(self
+            .0
             .map(
                 move |s| {
                     s.array()?
                         .to_struct(name_generator.clone())
-                        .map(|s| Some(s.into_series()))
+                        .map(|s| Some(s.into_column()))
                 },
                 GetOutput::map_dtype(move |dt: &DataType| {
                     let DataType::Array(inner, width) = dt else {
-                        panic!("Only array dtype is expected for `arr.to_struct`.")
+                        polars_bail!(InvalidOperation: "expected Array type, got: {}", dt)
                     };
 
                     let fields = (0..*width)
                         .map(|i| {
                             let name = arr_default_struct_name_gen(i);
-                            Field::from_owned(name, inner.as_ref().clone())
+                            Field::new(name, inner.as_ref().clone())
                         })
                         .collect();
                     Ok(DataType::Struct(fields))
                 }),
             )
-            .with_fmt("arr.to_struct")
+            .with_fmt("arr.to_struct"))
     }
 
     /// Shift every sub-array.
