@@ -6,6 +6,6 @@ pub struct CategoricalNameSpace(pub(crate) Expr);
 impl CategoricalNameSpace {
     pub fn get_categories(self) -> Expr {
         self.0
-            .map_private(CategoricalFunction::GetCategories.into())
+            .apply_private(CategoricalFunction::GetCategories.into())
     }
 }
