@@ -5,6 +5,7 @@ use polars_utils::pl_str::PlSmallStr;
 use polars_utils::IdxSize;
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
+use strum_macros::IntoStaticStr;
 
 use crate::dsl::Selector;
 
@@ -87,8 +88,9 @@ impl Default for WindowType {
     }
 }
 
-#[derive(Copy, Clone, Debug, PartialEq, Eq, Default, Hash)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, Default, Hash, IntoStaticStr)]
 #[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[strum(serialize_all = "snake_case")]
 pub enum WindowMapping {
     /// Map the group values to the position
     #[default]
@@ -106,7 +108,7 @@ pub enum WindowMapping {
 pub enum NestedType {
     #[cfg(feature = "dtype-array")]
     Array,
-    List,
+    // List,
 }
 
 #[derive(Clone, Default, Debug, PartialEq, Eq, Hash)]
