@@ -28,23 +28,36 @@ fn empty_df() -> IR {
     }
 }
 
-macro_rules! failed_input {
-    ($($t:tt)*) => {
-        failed_input_args!(stringify!($($t)*))
-    }
+fn validate_expression(
+    node: Node,
+    expr_arena: &Arena<AExpr>,
+    input_schema: &Schema,
+    operation_name: &str,
+) -> PolarsResult<()> {
+    let iter = aexpr_to_leaf_names_iter(node, expr_arena);
+    validate_columns_in_input(iter, input_schema, operation_name)
 }
-macro_rules! failed_input_args {
-    ($name:expr) => {
-        format!("'{}' input failed to resolve", $name).into()
-    };
+
+fn validate_expressions<N: Into<Node>, I: IntoIterator<Item = N>>(
+    nodes: I,
+    expr_arena: &Arena<AExpr>,
+    input_schema: &Schema,
+    operation_name: &str,
+) -> PolarsResult<()> {
+    let nodes = nodes.into_iter();
+
+    for node in nodes {
+        validate_expression(node.into(), expr_arena, input_schema, operation_name)?
+    }
+    Ok(())
 }
 
 macro_rules! failed_here {
     ($($t:tt)*) => {
-        format!("'{}' failed", stringify!($($t)*)).into()
+        format!("'{}'", stringify!($($t)*)).into()
     }
 }
-pub(super) use {failed_here, failed_input, failed_input_args};
+pub(super) use failed_here;
 
 pub fn to_alp(
     lp: DslPlan,
@@ -65,7 +78,29 @@ pub fn to_alp(
         opt_flags,
     };
 
-    to_alp_impl(lp, &mut ctxt)
+    match to_alp_impl(lp, &mut ctxt) {
+        Ok(out) => Ok(out),
+        Err(err) => {
+            if let Some(ir_until_then) = lp_arena.last_node() {
+                let node_name = if let PolarsError::Context { msg, .. } = &err {
+                    msg
+                } else {
+                    "THIS_NODE"
+                };
+                let plan = IRPlan::new(
+                    ir_until_then,
+                    std::mem::take(lp_arena),
+                    std::mem::take(expr_arena),
+                );
+                let location = format!("{}", plan.display());
+                Err(err.wrap_msg(|msg| {
+                    format!("{msg}\n\nResolved plan until failure:\n\n\t---> FAILED HERE RESOLVING {node_name} <---\n{location}")
+                }))
+            } else {
+                Err(err)
+            }
+        },
+    }
 }
 
 pub(super) struct DslConversionContext<'a> {
@@ -134,18 +169,31 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
                 let mut file_info = match &mut scan_type {
                     #[cfg(feature = "parquet")]
                     FileScan::Parquet {
+                        options,
                         cloud_options,
                         metadata,
-                        ..
                     } => {
-                        let (file_info, md) = scans::parquet_file_info(
-                            &sources,
-                            &file_options,
-                            cloud_options.as_ref(),
-                        )
-                        .map_err(|e| e.context(failed_here!(parquet scan)))?;
-                        *metadata = md;
-                        file_info
+                        if let Some(schema) = &options.schema {
+                            // We were passed a schema, we don't have to call `parquet_file_info`,
+                            // but this does mean we don't have `row_estimation` and `first_metadata`.
+                            FileInfo {
+                                schema: schema.clone(),
+                                reader_schema: Some(either::Either::Left(Arc::new(
+                                    schema.to_arrow(CompatLevel::newest()),
+                                ))),
+                                row_estimation: (None, 0),
+                            }
+                        } else {
+                            let (file_info, md) = scans::parquet_file_info(
+                                &sources,
+                                &file_options,
+                                cloud_options.as_ref(),
+                            )
+                            .map_err(|e| e.context(failed_here!(parquet scan)))?;
+
+                            *metadata = md;
+                            file_info
+                        }
                     },
                     #[cfg(feature = "ipc")]
                     FileScan::Ipc {
@@ -284,7 +332,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
                 .into_iter()
                 .map(|lp| to_alp_impl(lp, ctxt))
                 .collect::<PolarsResult<Vec<_>>>()
-                .map_err(|e| e.context(failed_input!(vertical concat)))?;
+                .map_err(|e| e.context(failed_here!(vertical concat)))?;
 
             if args.diagonal {
                 inputs =
@@ -293,7 +341,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
 
             if args.to_supertypes {
                 convert_utils::convert_st_union(&mut inputs, ctxt.lp_arena, ctxt.expr_arena)
-                    .map_err(|e| e.context(failed_input!(vertical concat)))?;
+                    .map_err(|e| e.context(failed_here!(vertical concat)))?;
             }
             let options = args.into();
             IR::Union { inputs, options }
@@ -303,7 +351,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
                 .into_iter()
                 .map(|lp| to_alp_impl(lp, ctxt))
                 .collect::<PolarsResult<Vec<_>>>()
-                .map_err(|e| e.context(failed_input!(horizontal concat)))?;
+                .map_err(|e| e.context(failed_here!(horizontal concat)))?;
 
             let schema = convert_utils::h_concat_schema(&inputs, ctxt.lp_arena)?;
 
@@ -315,13 +363,13 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
         },
         DslPlan::Filter { input, predicate } => {
             let mut input =
-                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_input!(filter)))?;
+                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_here!(filter)))?;
             let predicate = expand_filter(predicate, input, ctxt.lp_arena, ctxt.opt_flags)
                 .map_err(|e| e.context(failed_here!(filter)))?;
 
             let predicate_ae = to_expr_ir(predicate.clone(), ctxt.expr_arena)?;
 
-            return if is_streamable(predicate_ae.node(), ctxt.expr_arena, Context::Default) {
+            return if is_streamable(predicate_ae.node(), ctxt.expr_arena, Default::default()) {
                 // Split expression that are ANDed into multiple Filter nodes as the optimizer can then
                 // push them down independently. Especially if they refer columns from different tables
                 // this will be more performant.
@@ -365,7 +413,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
         },
         DslPlan::Slice { input, offset, len } => {
             let input =
-                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_input!(slice)))?;
+                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_here!(slice)))?;
             IR::Slice { input, offset, len }
         },
         DslPlan::DataFrameScan { df, schema } => IR::DataFrameScan {
@@ -380,7 +428,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
             options,
         } => {
             let input =
-                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_input!(select)))?;
+                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_here!(select)))?;
             let schema = ctxt.lp_arena.get(input).schema(ctxt.lp_arena);
             let (exprs, schema) = prepare_projection(expr, &schema, ctxt.opt_flags)
                 .map_err(|e| e.context(failed_here!(select)))?;
@@ -401,7 +449,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
                 options,
             };
 
-            return run_conversion(lp, ctxt, "select");
+            return run_conversion(lp, ctxt, "select").map_err(|e| e.context(failed_here!(select)));
         },
         DslPlan::Sort {
             input,
@@ -430,7 +478,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
             );
 
             let input =
-                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_input!(sort)))?;
+                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_here!(sort)))?;
 
             let mut expanded_cols = Vec::new();
             let mut nulls_last = Vec::new();
@@ -473,11 +521,11 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
                 sort_options,
             };
 
-            return run_conversion(lp, ctxt, "sort");
+            return run_conversion(lp, ctxt, "sort").map_err(|e| e.context(failed_here!(sort)));
         },
         DslPlan::Cache { input, id } => {
             let input =
-                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_input!(cache)))?;
+                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_here!(cache)))?;
             IR::Cache {
                 input,
                 id,
@@ -493,7 +541,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
             options,
         } => {
             let input =
-                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_input!(group_by)))?;
+                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_here!(group_by)))?;
 
             let (keys, aggs, schema) = resolve_group_by(
                 input,
@@ -527,7 +575,8 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
                 options,
             };
 
-            return run_conversion(lp, ctxt, "group_by");
+            return run_conversion(lp, ctxt, "group_by")
+                .map_err(|e| e.context(failed_here!(group_by)));
         },
         DslPlan::Join {
             input_left,
@@ -546,6 +595,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
                 options,
                 ctxt,
             )
+            .map_err(|e| e.context(failed_here!(join)))
         },
         DslPlan::HStack {
             input,
@@ -553,7 +603,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
             options,
         } => {
             let input = to_alp_impl(owned(input), ctxt)
-                .map_err(|e| e.context(failed_input!(with_columns)))?;
+                .map_err(|e| e.context(failed_here!(with_columns)))?;
             let (exprs, schema) =
                 resolve_with_columns(exprs, input, ctxt.lp_arena, ctxt.expr_arena, ctxt.opt_flags)
                     .map_err(|e| e.context(failed_here!(with_columns)))?;
@@ -570,7 +620,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
         },
         DslPlan::Distinct { input, options } => {
             let input =
-                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_input!(unique)))?;
+                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_here!(unique)))?;
             let input_schema = ctxt.lp_arena.get(input).schema(ctxt.lp_arena);
 
             let subset = options
@@ -587,9 +637,8 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
             IR::Distinct { input, options }
         },
         DslPlan::MapFunction { input, function } => {
-            let input = to_alp_impl(owned(input), ctxt).map_err(|e| {
-                e.context(failed_input_args!(format!("{}", function).to_lowercase()))
-            })?;
+            let input = to_alp_impl(owned(input), ctxt)
+                .map_err(|e| e.context(failed_here!(format!("{}", function).to_lowercase())))?;
             let input_schema = ctxt.lp_arena.get(input).schema(ctxt.lp_arena);
 
             match function {
@@ -598,7 +647,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
                     allow_empty,
                 } => {
                     let columns = expand_selectors(columns, &input_schema, &[])?;
-                    validate_columns_in_input(&columns, &input_schema, "explode")?;
+                    validate_columns_in_input(columns.as_ref(), &input_schema, "explode")?;
                     polars_ensure!(!columns.is_empty() || allow_empty, InvalidOperation: "no columns provided in explode");
                     if columns.is_empty() {
                         return Ok(input);
@@ -689,9 +738,9 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
                             |name| col(name.clone()).std(ddof),
                             &input_schema,
                         ),
-                        StatsFunction::Quantile { quantile, interpol } => stats_helper(
+                        StatsFunction::Quantile { quantile, method } => stats_helper(
                             |dt| dt.is_numeric(),
-                            |name| col(name.clone()).quantile(quantile.clone(), interpol),
+                            |name| col(name.clone()).quantile(quantile.clone(), method),
                             &input_schema,
                         ),
                         StatsFunction::Mean => stats_helper(
@@ -753,7 +802,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
         },
         DslPlan::ExtContext { input, contexts } => {
             let input = to_alp_impl(owned(input), ctxt)
-                .map_err(|e| e.context(failed_input!(with_context)))?;
+                .map_err(|e| e.context(failed_here!(with_context)))?;
             let contexts = contexts
                 .into_iter()
                 .map(|lp| to_alp_impl(lp, ctxt))
@@ -778,7 +827,7 @@ pub fn to_alp_impl(lp: DslPlan, ctxt: &mut DslConversionContext) -> PolarsResult
         },
         DslPlan::Sink { input, payload } => {
             let input =
-                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_input!(sink)))?;
+                to_alp_impl(owned(input), ctxt).map_err(|e| e.context(failed_here!(sink)))?;
             IR::Sink { input, payload }
         },
         DslPlan::IR { node, dsl, version } => {
@@ -955,8 +1004,10 @@ fn resolve_group_by(
             polars_ensure!(names.insert(name.clone()), duplicate = name)
         }
     }
-    let aggs = to_expr_irs(aggs, expr_arena)?;
     let keys = to_expr_irs(keys, expr_arena)?;
+    let aggs = to_expr_irs(aggs, expr_arena)?;
+    validate_expressions(&keys, expr_arena, current_schema, "group by")?;
+    validate_expressions(&aggs, expr_arena, current_schema, "group by")?;
 
     Ok((keys, aggs, Arc::new(schema)))
 }
