@@ -42,6 +42,7 @@ pub enum DslFunction {
     Rename {
         existing: Arc<[PlSmallStr]>,
         new: Arc<[PlSmallStr]>,
+        strict: bool,
     },
     Unnest(Vec<Selector>),
     Stats(StatsFunction),
@@ -71,7 +72,7 @@ pub enum StatsFunction {
     },
     Quantile {
         quantile: Expr,
-        interpol: QuantileInterpolOptions,
+        method: QuantileMethod,
     },
     Median,
     Mean,
@@ -80,11 +81,12 @@ pub enum StatsFunction {
     Max,
 }
 
-pub(crate) fn validate_columns_in_input<S: AsRef<str>>(
-    columns: &[S],
+pub(crate) fn validate_columns_in_input<S: AsRef<str>, I: IntoIterator<Item = S>>(
+    columns: I,
     input_schema: &Schema,
     operation_name: &str,
 ) -> PolarsResult<()> {
+    let columns = columns.into_iter();
     for c in columns {
         polars_ensure!(input_schema.contains(c.as_ref()), ColumnNotFound: "'{}' on column: '{}' is invalid\n\nSchema at this point: {:?}", operation_name, c.as_ref(), input_schema)
     }
@@ -119,10 +121,15 @@ impl DslFunction {
                 offset,
                 schema: Default::default(),
             },
-            DslFunction::Rename { existing, new } => {
+            DslFunction::Rename {
+                existing,
+                new,
+                strict,
+            } => {
                 let swapping = new.iter().any(|name| input_schema.get(name).is_some());
-                validate_columns_in_input(existing.as_ref(), input_schema, "rename")?;
-
+                if strict {
+                    validate_columns_in_input(existing.as_ref(), input_schema, "rename")?;
+                }
                 FunctionIR::Rename {
                     existing,
                     new,
