@@ -452,6 +452,12 @@ impl<'a, T: AsExpr> Display for ExprIRSliceDisplay<'a, T> {
     }
 }
 
+impl<'a, T: AsExpr> fmt::Debug for ExprIRSliceDisplay<'a, T> {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        Display::fmt(self, f)
+    }
+}
+
 impl<'a> Display for ExprIRDisplay<'a> {
     #[recursive]
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
@@ -587,6 +593,16 @@ impl<'a> Display for ExprIRDisplay<'a> {
                     Var(expr, _) => write!(f, "{}.var()", self.with_root(expr)),
                     Std(expr, _) => write!(f, "{}.std()", self.with_root(expr)),
                     Quantile { expr, .. } => write!(f, "{}.quantile()", self.with_root(expr)),
+                    #[cfg(feature = "bitwise")]
+                    Bitwise(expr, t) => {
+                        let t = match t {
+                            BitwiseAggFunction::And => "and",
+                            BitwiseAggFunction::Or => "or",
+                            BitwiseAggFunction::Xor => "xor",
+                        };
+
+                        write!(f, "{}.bitwise.{t}()", self.with_root(expr))
+                    },
                 }
             },
             Cast {
@@ -657,6 +673,12 @@ impl<'a> Display for ExprIRDisplay<'a> {
     }
 }
 
+impl<'a> fmt::Debug for ExprIRDisplay<'a> {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
+        Display::fmt(self, f)
+    }
+}
+
 pub(crate) struct ColumnsDisplay<'a>(pub(crate) &'a Schema);
 
 impl fmt::Display for ColumnsDisplay<'_> {
