@@ -181,6 +181,11 @@ enum VisitRecord {
 fn skip_pre_visit(ae: &AExpr, is_groupby: bool) -> bool {
     match ae {
         AExpr::Window { .. } => true,
+        #[cfg(feature = "dtype-struct")]
+        AExpr::Function {
+            function: FunctionExpr::AsStruct,
+            ..
+        } => true,
         AExpr::Ternary { .. } => is_groupby,
         _ => false,
     }
