@@ -7,6 +7,7 @@ mod delay_rechunk;
 
 mod cluster_with_columns;
 mod collapse_and_project;
+mod collapse_joins;
 mod collect_members;
 mod count_star;
 #[cfg(feature = "cse")]
@@ -82,6 +83,7 @@ pub fn optimize(
 
     // get toggle values
     let cluster_with_columns = opt_state.contains(OptFlags::CLUSTER_WITH_COLUMNS);
+    let collapse_joins = opt_state.contains(OptFlags::COLLAPSE_JOINS);
     let predicate_pushdown = opt_state.contains(OptFlags::PREDICATE_PUSHDOWN);
     let projection_pushdown = opt_state.contains(OptFlags::PROJECTION_PUSHDOWN);
     let simplify_expr = opt_state.contains(OptFlags::SIMPLIFY_EXPR);
@@ -164,6 +166,11 @@ pub fn optimize(
         cluster_with_columns::optimize(lp_top, lp_arena, expr_arena)
     }
 
+    // Make sure it is after predicate pushdown
+    if collapse_joins && members.has_filter_with_join_input {
+        collapse_joins::optimize(lp_top, lp_arena, expr_arena)
+    }
+
     // Make sure its before slice pushdown.
     if fast_projection {
         rules.push(Box::new(SimpleProjectionAndCollapse::new(eager)));
