@@ -344,6 +344,7 @@ impl ProjectionPushDown {
                 projections_seen,
                 lp_arena,
                 expr_arena,
+                false,
             ),
             SimpleProjection { columns, input, .. } => {
                 let exprs = names_to_expr_irs(columns.iter_names_cloned(), expr_arena);
@@ -356,6 +357,7 @@ impl ProjectionPushDown {
                     projections_seen,
                     lp_arena,
                     expr_arena,
+                    true,
                 )
             },
             DataFrameScan {
@@ -509,6 +511,21 @@ impl ProjectionPushDown {
                         file_options.row_index = None;
                     }
                 };
+
+                if let Some(col_name) = &file_options.include_file_paths {
+                    if output_schema
+                        .as_ref()
+                        .map_or(false, |schema| !schema.contains(col_name))
+                    {
+                        // Need to remove it from the input schema so
+                        // that projection indices are correct.
+                        let mut file_schema = Arc::unwrap_or_clone(file_info.schema);
+                        file_schema.shift_remove(col_name);
+                        file_info.schema = Arc::new(file_schema);
+                        file_options.include_file_paths = None;
+                    }
+                };
+
                 let lp = Scan {
                     sources,
                     file_info,
