@@ -26,7 +26,7 @@ impl PyBatchedCsv {
         projection, separator, rechunk, columns, encoding, n_threads, path, overwrite_dtype,
         overwrite_dtype_slice, low_memory, comment_prefix, quote_char, null_values,
         missing_utf8_is_empty_string, try_parse_dates, skip_rows_after_header, row_index,
-        sample_size, eol_char, raise_if_empty, truncate_ragged_lines, decimal_comma)
+        eol_char, raise_if_empty, truncate_ragged_lines, decimal_comma)
     )]
     fn new(
         infer_schema_length: Option<usize>,
@@ -52,7 +52,6 @@ impl PyBatchedCsv {
         try_parse_dates: bool,
         skip_rows_after_header: usize,
         row_index: Option<(String, IdxSize)>,
-        sample_size: usize,
         eol_char: &str,
         raise_if_empty: bool,
         truncate_ragged_lines: bool,
@@ -108,7 +107,6 @@ impl PyBatchedCsv {
             .with_low_memory(low_memory)
             .with_skip_rows_after_header(skip_rows_after_header)
             .with_row_index(row_index)
-            .with_sample_size(sample_size)
             .with_raise_if_empty(raise_if_empty)
             .with_parse_options(
                 CsvParseOptions::default()
