@@ -1,8 +1,8 @@
 use polars::datatypes::TimeUnit;
+#[cfg(feature = "iejoin")]
+use polars::prelude::InequalityOperator;
 use polars::series::ops::NullBehavior;
-use polars_core::prelude::{NonExistent, QuantileInterpolOptions};
 use polars_core::series::IsSorted;
-use polars_ops::prelude::ClosedInterval;
 use polars_ops::series::InterpolationMethod;
 #[cfg(feature = "search_sorted")]
 use polars_ops::series::SearchSortedSide;
@@ -14,6 +14,7 @@ use polars_plan::prelude::{
     WindowMapping, WindowType,
 };
 use polars_time::prelude::RollingGroupOptions;
+use polars_time::{Duration, DynamicGroupOptions};
 use pyo3::exceptions::PyNotImplementedError;
 use pyo3::prelude::*;
 
@@ -42,18 +43,6 @@ pub struct Literal {
     dtype: PyObject,
 }
 
-impl IntoPy<PyObject> for Wrap<ClosedInterval> {
-    fn into_py(self, py: Python<'_>) -> PyObject {
-        match self.0 {
-            ClosedInterval::Both => "both",
-            ClosedInterval::Left => "left",
-            ClosedInterval::Right => "right",
-            ClosedInterval::None => "none",
-        }
-        .into_py(py)
-    }
-}
-
 #[pyclass(name = "Operator")]
 #[derive(Copy, Clone)]
 pub enum PyOperator {
@@ -114,6 +103,19 @@ impl IntoPy<PyObject> for Wrap<Operator> {
     }
 }
 
+#[cfg(feature = "iejoin")]
+impl IntoPy<PyObject> for Wrap<InequalityOperator> {
+    fn into_py(self, py: Python<'_>) -> PyObject {
+        match self.0 {
+            InequalityOperator::Lt => PyOperator::Lt,
+            InequalityOperator::LtEq => PyOperator::LtEq,
+            InequalityOperator::Gt => PyOperator::Gt,
+            InequalityOperator::GtEq => PyOperator::GtEq,
+        }
+        .into_py(py)
+    }
+}
+
 #[pyclass(name = "StringFunction")]
 #[derive(Copy, Clone)]
 pub enum PyStringFunction {
@@ -159,6 +161,7 @@ pub enum PyStringFunction {
     ZFill,
     ContainsMany,
     ReplaceMany,
+    EscapeRegex,
 }
 
 #[pymethods]
@@ -388,15 +391,25 @@ pub struct PyWindowMapping {
 impl PyWindowMapping {
     #[getter]
     fn kind(&self, py: Python<'_>) -> PyResult<PyObject> {
-        let result = match self.inner {
-            WindowMapping::GroupsToRows => "groups_to_rows".to_object(py),
-            WindowMapping::Explode => "explode".to_object(py),
-            WindowMapping::Join => "join".to_object(py),
-        };
+        let result: &str = self.inner.into();
         Ok(result.into_py(py))
     }
 }
 
+impl IntoPy<PyObject> for Wrap<Duration> {
+    fn into_py(self, py: Python<'_>) -> PyObject {
+        (
+            self.0.months(),
+            self.0.weeks(),
+            self.0.days(),
+            self.0.nanoseconds(),
+            self.0.parsed_int,
+            self.0.negative(),
+        )
+            .into_py(py)
+    }
+}
+
 #[pyclass(name = "RollingGroupOptions")]
 pub struct PyRollingGroupOptions {
     inner: RollingGroupOptions,
@@ -411,41 +424,68 @@ impl PyRollingGroupOptions {
 
     #[getter]
     fn period(&self, py: Python<'_>) -> PyResult<PyObject> {
-        let result = vec![
-            self.inner.period.months().to_object(py),
-            self.inner.period.weeks().to_object(py),
-            self.inner.period.days().to_object(py),
-            self.inner.period.nanoseconds().to_object(py),
-            self.inner.period.parsed_int.to_object(py),
-            self.inner.period.negative().to_object(py),
-        ]
-        .into_py(py);
-        Ok(result)
+        Ok(Wrap(self.inner.period).into_py(py))
     }
 
     #[getter]
     fn offset(&self, py: Python<'_>) -> PyResult<PyObject> {
-        let result = vec![
-            self.inner.offset.months().to_object(py),
-            self.inner.offset.weeks().to_object(py),
-            self.inner.offset.days().to_object(py),
-            self.inner.offset.nanoseconds().to_object(py),
-            self.inner.offset.parsed_int.to_object(py),
-            self.inner.offset.negative().to_object(py),
-        ]
-        .into_py(py);
-        Ok(result)
+        Ok(Wrap(self.inner.offset).into_py(py))
     }
 
     #[getter]
     fn closed_window(&self, py: Python<'_>) -> PyResult<PyObject> {
-        let result = match self.inner.closed_window {
-            polars::time::ClosedWindow::Left => "left".to_object(py),
-            polars::time::ClosedWindow::Right => "right".to_object(py),
-            polars::time::ClosedWindow::Both => "both".to_object(py),
-            polars::time::ClosedWindow::None => "none".to_object(py),
-        };
-        Ok(result.into_py(py))
+        let result: &str = self.inner.closed_window.into();
+        Ok(result.to_object(py))
+    }
+}
+
+#[pyclass(name = "DynamicGroupOptions")]
+pub struct PyDynamicGroupOptions {
+    inner: DynamicGroupOptions,
+}
+
+#[pymethods]
+impl PyDynamicGroupOptions {
+    #[getter]
+    fn index_column(&self, py: Python<'_>) -> PyResult<PyObject> {
+        Ok(self.inner.index_column.to_object(py))
+    }
+
+    #[getter]
+    fn every(&self, py: Python<'_>) -> PyResult<PyObject> {
+        Ok(Wrap(self.inner.every).into_py(py))
+    }
+
+    #[getter]
+    fn period(&self, py: Python<'_>) -> PyResult<PyObject> {
+        Ok(Wrap(self.inner.period).into_py(py))
+    }
+
+    #[getter]
+    fn offset(&self, py: Python<'_>) -> PyResult<PyObject> {
+        Ok(Wrap(self.inner.offset).into_py(py))
+    }
+
+    #[getter]
+    fn label(&self, py: Python<'_>) -> PyResult<PyObject> {
+        let result: &str = self.inner.label.into();
+        Ok(result.to_object(py))
+    }
+
+    #[getter]
+    fn include_boundaries(&self, py: Python<'_>) -> PyResult<PyObject> {
+        Ok(self.inner.include_boundaries.into_py(py))
+    }
+
+    #[getter]
+    fn closed_window(&self, py: Python<'_>) -> PyResult<PyObject> {
+        let result: &str = self.inner.closed_window.into();
+        Ok(result.to_object(py))
+    }
+    #[getter]
+    fn start_by(&self, py: Python<'_>) -> PyResult<PyObject> {
+        let result: &str = self.inner.start_by.into();
+        Ok(result.to_object(py))
     }
 }
 
@@ -470,6 +510,14 @@ impl PyGroupbyOptions {
             .map_or_else(|| py.None(), |f| f.to_object(py)))
     }
 
+    #[getter]
+    fn dynamic(&self, py: Python<'_>) -> PyResult<PyObject> {
+        Ok(self.inner.dynamic.as_ref().map_or_else(
+            || py.None(),
+            |f| PyDynamicGroupOptions { inner: f.clone() }.into_py(py),
+        ))
+    }
+
     #[getter]
     fn rolling(&self, py: Python<'_>) -> PyResult<PyObject> {
         Ok(self.inner.rolling.as_ref().map_or_else(
@@ -685,18 +733,11 @@ pub(crate) fn into_py(py: Python<'_>, expr: &AExpr) -> PyResult<PyObject> {
             IRAggExpr::Quantile {
                 expr,
                 quantile,
-                interpol,
+                method: interpol,
             } => Agg {
                 name: "quantile".to_object(py),
                 arguments: vec![expr.0, quantile.0],
-                options: match interpol {
-                    QuantileInterpolOptions::Nearest => "nearest",
-                    QuantileInterpolOptions::Lower => "lower",
-                    QuantileInterpolOptions::Higher => "higher",
-                    QuantileInterpolOptions::Midpoint => "midpoint",
-                    QuantileInterpolOptions::Linear => "linear",
-                }
-                .to_object(py),
+                options: Into::<&str>::into(interpol).to_object(py),
             },
             IRAggExpr::Sum(n) => Agg {
                 name: "sum".to_object(py),
@@ -723,6 +764,11 @@ pub(crate) fn into_py(py: Python<'_>, expr: &AExpr) -> PyResult<PyObject> {
                 arguments: vec![n.0],
                 options: py.None(),
             },
+            IRAggExpr::Bitwise(n, f) => Agg {
+                name: "bitwise".to_object(py),
+                arguments: vec![n.0],
+                options: Into::<&str>::into(f).to_object(py),
+            },
         }
         .into_py(py),
         AExpr::Ternary {
@@ -758,6 +804,9 @@ pub(crate) fn into_py(py: Python<'_>, expr: &AExpr) -> PyResult<PyObject> {
                 FunctionExpr::ListExpr(_) => {
                     return Err(PyNotImplementedError::new_err("list expr"))
                 },
+                FunctionExpr::Bitwise(_) => {
+                    return Err(PyNotImplementedError::new_err("bitwise expr"))
+                },
                 FunctionExpr::StringExpr(strfun) => match strfun {
                     StringFunction::ConcatHorizontal {
                         delimiter,
@@ -924,6 +973,9 @@ pub(crate) fn into_py(py: Python<'_>, expr: &AExpr) -> PyResult<PyObject> {
                     StringFunction::ExtractMany { .. } => {
                         return Err(PyNotImplementedError::new_err("extract_many"))
                     },
+                    StringFunction::EscapeRegex => {
+                        (PyStringFunction::EscapeRegex.into_py(py),).to_object(py)
+                    },
                 },
                 FunctionExpr::StructExpr(_) => {
                     return Err(PyNotImplementedError::new_err("struct expr"))
@@ -1002,10 +1054,7 @@ pub(crate) fn into_py(py: Python<'_>, expr: &AExpr) -> PyResult<PyObject> {
                         time_zone
                             .as_ref()
                             .map_or_else(|| py.None(), |s| s.to_object(py)),
-                        match non_existent {
-                            NonExistent::Null => "nullify",
-                            NonExistent::Raise => "raise",
-                        },
+                        Into::<&str>::into(non_existent),
                     )
                         .into_py(py),
                     TemporalFunction::Combine(time_unit) => {
@@ -1045,7 +1094,7 @@ pub(crate) fn into_py(py: Python<'_>, expr: &AExpr) -> PyResult<PyObject> {
                     BooleanFunction::IsUnique => (PyBooleanFunction::IsUnique,).into_py(py),
                     BooleanFunction::IsDuplicated => (PyBooleanFunction::IsDuplicated,).into_py(py),
                     BooleanFunction::IsBetween { closed } => {
-                        (PyBooleanFunction::IsBetween, Wrap(*closed)).into_py(py)
+                        (PyBooleanFunction::IsBetween, Into::<&str>::into(closed)).into_py(py)
                     },
                     #[cfg(feature = "is_in")]
                     BooleanFunction::IsIn => (PyBooleanFunction::IsIn,).into_py(py),
@@ -1138,6 +1187,9 @@ pub(crate) fn into_py(py: Python<'_>, expr: &AExpr) -> PyResult<PyObject> {
                     RollingFunction::Skew(_, _) => {
                         return Err(PyNotImplementedError::new_err("rolling skew"))
                     },
+                    RollingFunction::CorrCov { .. } => {
+                        return Err(PyNotImplementedError::new_err("rolling cor_cov"))
+                    },
                 },
                 FunctionExpr::RollingExprBy(rolling) => match rolling {
                     RollingFunctionBy::MinBy(_) => {
@@ -1169,12 +1221,12 @@ pub(crate) fn into_py(py: Python<'_>, expr: &AExpr) -> PyResult<PyObject> {
                 FunctionExpr::Mode => ("mode",).to_object(py),
                 FunctionExpr::Skew(bias) => ("skew", bias).to_object(py),
                 FunctionExpr::Kurtosis(fisher, bias) => ("kurtosis", fisher, bias).to_object(py),
-                FunctionExpr::Reshape(_, _) => {
-                    return Err(PyNotImplementedError::new_err("reshape"))
-                },
+                #[cfg(feature = "dtype-array")]
+                FunctionExpr::Reshape(_) => return Err(PyNotImplementedError::new_err("reshape")),
                 #[cfg(feature = "repeat_by")]
                 FunctionExpr::RepeatBy => ("repeat_by",).to_object(py),
                 FunctionExpr::ArgUnique => ("arg_unique",).to_object(py),
+                FunctionExpr::Repeat => ("repeat",).to_object(py),
                 FunctionExpr::Rank {
                     options: _,
                     seed: _,
