@@ -2,6 +2,7 @@ use std::fmt::Write;
 
 use polars_plan::plans::expr_ir::ExprIR;
 use polars_plan::plans::{AExpr, EscapeLabel, FileScan, ScanSourcesDisplay};
+use polars_plan::prelude::FileType;
 use polars_utils::arena::Arena;
 use polars_utils::itertools::Itertools;
 use slotmap::{Key, SecondaryMap, SlotMap};
@@ -59,6 +60,14 @@ fn visualize_plan_rec(
                 from_ref(input),
             )
         },
+        PhysNodeKind::WithRowIndex {
+            input,
+            name,
+            offset,
+        } => (
+            format!("with-row-index\\nname: {name}\\noffset: {offset:?}"),
+            from_ref(input),
+        ),
         PhysNodeKind::InputIndependentSelect { selectors } => (
             format!(
                 "input-independent-select\\n{}",
@@ -87,6 +96,14 @@ fn visualize_plan_rec(
             from_ref(input),
         ),
         PhysNodeKind::InMemorySink { input } => ("in-memory-sink".to_string(), from_ref(input)),
+        PhysNodeKind::FileSink {
+            input, file_type, ..
+        } => match file_type {
+            FileType::Parquet(_) => ("parquet-sink".to_string(), from_ref(input)),
+            FileType::Ipc(_) => ("ipc-sink".to_string(), from_ref(input)),
+            FileType::Csv(_) => ("csv-sink".to_string(), from_ref(input)),
+            FileType::Json(_) => ("json-sink".to_string(), from_ref(input)),
+        },
         PhysNodeKind::InMemoryMap { input, map: _ } => {
             ("in-memory-map".to_string(), from_ref(input))
         },
@@ -175,6 +192,17 @@ fn visualize_plan_rec(
 
             (out, &[][..])
         },
+        PhysNodeKind::GroupBy { input, key, aggs } => {
+            let label = "group-by";
+            (
+                format!(
+                    "{label}\\nkey:\\n{}\\naggs:\\n{}",
+                    fmt_exprs(key, expr_arena),
+                    fmt_exprs(aggs, expr_arena)
+                ),
+                from_ref(input),
+            )
+        },
     };
 
     out.push(format!(
