@@ -87,14 +87,24 @@ pub(crate) fn is_elementwise(
             function: _,
             output_type: _,
             options,
-        }
-        | AExpr::Function {
-            input,
-            function: _,
-            options,
         } => {
             options.is_elementwise() && input.iter().all(|e| is_elementwise(e.node(), arena, cache))
         },
+        AExpr::Function {
+            input,
+            function,
+            options,
+        } => {
+            match function {
+                // Non-strict strptime must be done in-memory to ensure the format
+                // is consistent across the entire dataframe.
+                FunctionExpr::StringExpr(StringFunction::Strptime(_, opts)) => opts.strict,
+                _ => {
+                    options.is_elementwise()
+                        && input.iter().all(|e| is_elementwise(e.node(), arena, cache))
+                },
+            }
+        },
 
         AExpr::Window { .. } => false,
         AExpr::Slice { .. } => false,
@@ -338,7 +348,7 @@ fn build_fallback_node_with_ctx(
                 expr,
                 Context::Default,
                 ctx.expr_arena,
-                None,
+                &ctx.phys_sm[input_node].output_schema,
                 &mut conv_state,
             )
         })
@@ -563,7 +573,9 @@ fn lower_exprs_with_ctx(
                     ..
                 }
                 | IRAggExpr::Sum(ref mut inner)
-                | IRAggExpr::Mean(ref mut inner) => {
+                | IRAggExpr::Mean(ref mut inner)
+                | IRAggExpr::Var(ref mut inner, _ /* ddof */)
+                | IRAggExpr::Std(ref mut inner, _ /* ddof */) => {
                     let (trans_input, trans_exprs) = lower_exprs_with_ctx(input, &[*inner], ctx)?;
                     *inner = trans_exprs[0];
 
@@ -586,13 +598,17 @@ fn lower_exprs_with_ctx(
                 | IRAggExpr::Implode(_)
                 | IRAggExpr::Quantile { .. }
                 | IRAggExpr::Count(_, _)
-                | IRAggExpr::Std(_, _)
-                | IRAggExpr::Var(_, _)
                 | IRAggExpr::AggGroups(_) => {
                     let out_name = unique_column_name();
                     fallback_subset.push(ExprIR::new(expr, OutputName::Alias(out_name.clone())));
                     transformed_exprs.push(ctx.expr_arena.add(AExpr::Column(out_name)));
                 },
+                #[cfg(feature = "bitwise")]
+                IRAggExpr::Bitwise(_, _) => {
+                    let out_name = unique_column_name();
+                    fallback_subset.push(ExprIR::new(expr, OutputName::Alias(out_name.clone())));
+                    transformed_exprs.push(ctx.expr_arena.add(AExpr::Column(out_name)));
+                },
             },
             AExpr::Len => {
                 let out_name = unique_column_name();
@@ -648,7 +664,7 @@ fn lower_exprs_with_ctx(
 
 /// Computes the schema that selecting the given expressions on the input schema
 /// would result in.
-fn compute_output_schema(
+pub fn compute_output_schema(
     input_schema: &Schema,
     exprs: &[ExprIR],
     expr_arena: &Arena<AExpr>,
