@@ -177,7 +177,7 @@ impl Window {
         tu: TimeUnit,
         tz: Option<&'a Tz>,
         start_by: StartBy,
-    ) -> PolarsResult<BoundsIter> {
+    ) -> PolarsResult<BoundsIter<'a>> {
         BoundsIter::new(*self, closed_window, boundary, tu, tz, start_by)
     }
 }
