@@ -45,10 +45,7 @@ impl<T> UnitVec<T> {
     #[inline]
     pub fn new() -> Self {
         // This is optimized away, all const.
-        assert!(
-            std::mem::size_of::<T>() <= std::mem::size_of::<*mut T>()
-                && std::mem::align_of::<T>() <= std::mem::align_of::<*mut T>()
-        );
+        assert!(size_of::<T>() <= size_of::<*mut T>() && align_of::<T>() <= align_of::<*mut T>());
         Self {
             len: 0,
             capacity: NonZeroUsize::new(1).unwrap(),
@@ -71,6 +68,11 @@ impl<T> UnitVec<T> {
         self.capacity.get()
     }
 
+    #[inline(always)]
+    pub fn clear(&mut self) {
+        self.len = 0;
+    }
+
     #[inline(always)]
     pub fn push(&mut self, idx: T) {
         if self.len == self.capacity.get() {
