@@ -24,7 +24,11 @@ fn nulls() {
         ArrowDataType::Float64,
         ArrowDataType::Utf8,
         ArrowDataType::Binary,
-        ArrowDataType::List(Box::new(Field::new("a", ArrowDataType::Binary, true))),
+        ArrowDataType::List(Box::new(Field::new(
+            "a".into(),
+            ArrowDataType::Binary,
+            true,
+        ))),
     ];
     let a = datatypes
         .into_iter()
@@ -34,12 +38,12 @@ fn nulls() {
     // unions' null count is always 0
     let datatypes = vec![
         ArrowDataType::Union(
-            vec![Field::new("a", ArrowDataType::Binary, true)],
+            vec![Field::new("a".into(), ArrowDataType::Binary, true)],
             None,
             UnionMode::Dense,
         ),
         ArrowDataType::Union(
-            vec![Field::new("a", ArrowDataType::Binary, true)],
+            vec![Field::new("a".into(), ArrowDataType::Binary, true)],
             None,
             UnionMode::Sparse,
         ),
@@ -57,23 +61,27 @@ fn empty() {
         ArrowDataType::Float64,
         ArrowDataType::Utf8,
         ArrowDataType::Binary,
-        ArrowDataType::List(Box::new(Field::new("a", ArrowDataType::Binary, true))),
         ArrowDataType::List(Box::new(Field::new(
-            "a",
-            ArrowDataType::Extension("ext".to_owned(), Box::new(ArrowDataType::Int32), None),
+            "a".into(),
+            ArrowDataType::Binary,
+            true,
+        ))),
+        ArrowDataType::List(Box::new(Field::new(
+            "a".into(),
+            ArrowDataType::Extension("ext".into(), Box::new(ArrowDataType::Int32), None),
             true,
         ))),
         ArrowDataType::Union(
-            vec![Field::new("a", ArrowDataType::Binary, true)],
+            vec![Field::new("a".into(), ArrowDataType::Binary, true)],
             None,
             UnionMode::Sparse,
         ),
         ArrowDataType::Union(
-            vec![Field::new("a", ArrowDataType::Binary, true)],
+            vec![Field::new("a".into(), ArrowDataType::Binary, true)],
             None,
             UnionMode::Dense,
         ),
-        ArrowDataType::Struct(vec![Field::new("a", ArrowDataType::Int32, true)]),
+        ArrowDataType::Struct(vec![Field::new("a".into(), ArrowDataType::Int32, true)]),
     ];
     let a = datatypes.into_iter().all(|x| new_empty_array(x).len() == 0);
     assert!(a);
@@ -86,25 +94,29 @@ fn empty_extension() {
         ArrowDataType::Float64,
         ArrowDataType::Utf8,
         ArrowDataType::Binary,
-        ArrowDataType::List(Box::new(Field::new("a", ArrowDataType::Binary, true))),
+        ArrowDataType::List(Box::new(Field::new(
+            "a".into(),
+            ArrowDataType::Binary,
+            true,
+        ))),
         ArrowDataType::Union(
-            vec![Field::new("a", ArrowDataType::Binary, true)],
+            vec![Field::new("a".into(), ArrowDataType::Binary, true)],
             None,
             UnionMode::Sparse,
         ),
         ArrowDataType::Union(
-            vec![Field::new("a", ArrowDataType::Binary, true)],
+            vec![Field::new("a".into(), ArrowDataType::Binary, true)],
             None,
             UnionMode::Dense,
         ),
-        ArrowDataType::Struct(vec![Field::new("a", ArrowDataType::Int32, true)]),
+        ArrowDataType::Struct(vec![Field::new("a".into(), ArrowDataType::Int32, true)]),
     ];
     let a = datatypes
         .into_iter()
-        .map(|dt| ArrowDataType::Extension("ext".to_owned(), Box::new(dt), None))
+        .map(|dt| ArrowDataType::Extension("ext".into(), Box::new(dt), None))
         .all(|x| {
             let a = new_empty_array(x);
-            a.len() == 0 && matches!(a.data_type(), ArrowDataType::Extension(_, _, _))
+            a.len() == 0 && matches!(a.dtype(), ArrowDataType::Extension(_, _, _))
         });
     assert!(a);
 }
@@ -116,7 +128,11 @@ fn test_clone() {
         ArrowDataType::Float64,
         ArrowDataType::Utf8,
         ArrowDataType::Binary,
-        ArrowDataType::List(Box::new(Field::new("a", ArrowDataType::Binary, true))),
+        ArrowDataType::List(Box::new(Field::new(
+            "a".into(),
+            ArrowDataType::Binary,
+            true,
+        ))),
     ];
     let a = datatypes
         .into_iter()
