@@ -62,7 +62,7 @@ fn prep_schema(array: &dyn Array) -> ArrowSchemaRef {
 fn write_boolean() -> PolarsResult<()> {
     let array = BooleanArray::from([Some(true), Some(false), None, Some(true)]).boxed();
     let schema = prep_schema(array.as_ref());
-    let columns = RecordBatchT::try_new(vec![array])?;
+    let columns = RecordBatchT::try_new(4, vec![array])?;
     round_trip(columns, schema, None, Some(Compression::ZSTD))
 }
 
@@ -72,7 +72,7 @@ fn write_sliced_utf8() -> PolarsResult<()> {
         .sliced(1, 1)
         .boxed();
     let schema = prep_schema(array.as_ref());
-    let columns = RecordBatchT::try_new(vec![array])?;
+    let columns = RecordBatchT::try_new(array.len(), vec![array])?;
     round_trip(columns, schema, None, Some(Compression::ZSTD))
 }
 
@@ -80,6 +80,6 @@ fn write_sliced_utf8() -> PolarsResult<()> {
 fn write_binview() -> PolarsResult<()> {
     let array = Utf8ViewArray::from_slice([Some("foo"), Some("bar"), None, Some("hamlet")]).boxed();
     let schema = prep_schema(array.as_ref());
-    let columns = RecordBatchT::try_new(vec![array])?;
+    let columns = RecordBatchT::try_new(array.len(), vec![array])?;
     round_trip(columns, schema, None, Some(Compression::ZSTD))
 }
