@@ -39,13 +39,13 @@ fn test_chunked_left_join() -> PolarsResult<()> {
 }
 
 fn create_frames() -> (DataFrame, DataFrame) {
-    let s0 = Series::new("days", &[0, 1, 2]);
-    let s1 = Series::new("temp", &[22.1, 19.9, 7.]);
-    let s2 = Series::new("rain", &[0.2, 0.1, 0.3]);
+    let s0 = Column::new("days".into(), &[0, 1, 2]);
+    let s1 = Column::new("temp".into(), &[22.1, 19.9, 7.]);
+    let s2 = Column::new("rain".into(), &[0.2, 0.1, 0.3]);
     let temp = DataFrame::new(vec![s0, s1, s2]).unwrap();
 
-    let s0 = Series::new("days", &[1, 2, 3, 1]);
-    let s1 = Series::new("rain", &[0.1, 0.2, 0.3, 0.4]);
+    let s0 = Column::new("days".into(), &[1, 2, 3, 1]);
+    let s1 = Column::new("rain".into(), &[0.1, 0.2, 0.3, 0.4]);
     let rain = DataFrame::new(vec![s0, s1]).unwrap();
     (temp, rain)
 }
@@ -59,10 +59,10 @@ fn test_inner_join() {
         std::env::set_var("POLARS_MAX_THREADS", format!("{}", i));
         let joined = temp.inner_join(&rain, ["days"], ["days"]).unwrap();
 
-        let join_col_days = Series::new("days", &[1, 2, 1]);
-        let join_col_temp = Series::new("temp", &[19.9, 7., 19.9]);
-        let join_col_rain = Series::new("rain", &[0.1, 0.3, 0.1]);
-        let join_col_rain_right = Series::new("rain_right", [0.1, 0.2, 0.4].as_ref());
+        let join_col_days = Column::new("days".into(), &[1, 2, 1]);
+        let join_col_temp = Column::new("temp".into(), &[19.9, 7., 19.9]);
+        let join_col_rain = Column::new("rain".into(), &[0.1, 0.3, 0.1]);
+        let join_col_rain_right = Column::new("rain_right".into(), [0.1, 0.2, 0.4].as_ref());
         let true_df = DataFrame::new(vec![
             join_col_days,
             join_col_temp,
@@ -81,31 +81,45 @@ fn test_inner_join() {
 fn test_left_join() {
     for i in 1..8 {
         std::env::set_var("POLARS_MAX_THREADS", format!("{}", i));
-        let s0 = Series::new("days", &[0, 1, 2, 3, 4]);
-        let s1 = Series::new("temp", &[22.1, 19.9, 7., 2., 3.]);
+        let s0 = Column::new("days".into(), &[0, 1, 2, 3, 4]);
+        let s1 = Column::new("temp".into(), &[22.1, 19.9, 7., 2., 3.]);
         let temp = DataFrame::new(vec![s0, s1]).unwrap();
 
-        let s0 = Series::new("days", &[1, 2]);
-        let s1 = Series::new("rain", &[0.1, 0.2]);
+        let s0 = Column::new("days".into(), &[1, 2]);
+        let s1 = Column::new("rain".into(), &[0.1, 0.2]);
         let rain = DataFrame::new(vec![s0, s1]).unwrap();
         let joined = temp.left_join(&rain, ["days"], ["days"]).unwrap();
         assert_eq!(
-            (joined.column("rain").unwrap().sum::<f32>().unwrap() * 10.).round(),
+            (joined
+                .column("rain")
+                .unwrap()
+                .as_materialized_series()
+                .sum::<f32>()
+                .unwrap()
+                * 10.)
+                .round(),
             3.
         );
         assert_eq!(joined.column("rain").unwrap().null_count(), 3);
 
         // test join on string
-        let s0 = Series::new("days", &["mo", "tue", "wed", "thu", "fri"]);
-        let s1 = Series::new("temp", &[22.1, 19.9, 7., 2., 3.]);
+        let s0 = Column::new("days".into(), &["mo", "tue", "wed", "thu", "fri"]);
+        let s1 = Column::new("temp".into(), &[22.1, 19.9, 7., 2., 3.]);
         let temp = DataFrame::new(vec![s0, s1]).unwrap();
 
-        let s0 = Series::new("days", &["tue", "wed"]);
-        let s1 = Series::new("rain", &[0.1, 0.2]);
+        let s0 = Column::new("days".into(), &["tue", "wed"]);
+        let s1 = Column::new("rain".into(), &[0.1, 0.2]);
         let rain = DataFrame::new(vec![s0, s1]).unwrap();
         let joined = temp.left_join(&rain, ["days"], ["days"]).unwrap();
         assert_eq!(
-            (joined.column("rain").unwrap().sum::<f32>().unwrap() * 10.).round(),
+            (joined
+                .column("rain")
+                .unwrap()
+                .as_materialized_series()
+                .sum::<f32>()
+                .unwrap()
+                * 10.)
+                .round(),
             3.
         );
         assert_eq!(joined.column("rain").unwrap().null_count(), 3);
@@ -123,7 +137,14 @@ fn test_full_outer_join() -> PolarsResult<()> {
         JoinArgs::new(JoinType::Full).with_coalesce(JoinCoalesce::CoalesceColumns),
     )?;
     assert_eq!(joined.height(), 5);
-    assert_eq!(joined.column("days")?.sum::<i32>().unwrap(), 7);
+    assert_eq!(
+        joined
+            .column("days")?
+            .as_materialized_series()
+            .sum::<i32>()
+            .unwrap(),
+        7
+    );
 
     let df_left = df!(
             "a"=> ["a", "b", "a", "z"],
@@ -152,12 +173,16 @@ fn test_full_outer_join() -> PolarsResult<()> {
 fn test_join_with_nulls() {
     let dts = &[20, 21, 22, 23, 24, 25, 27, 28];
     let vals = &[1.2, 2.4, 4.67, 5.8, 4.4, 3.6, 7.6, 6.5];
-    let df = DataFrame::new(vec![Series::new("date", dts), Series::new("val", vals)]).unwrap();
+    let df = DataFrame::new(vec![
+        Column::new("date".into(), dts),
+        Column::new("val".into(), vals),
+    ])
+    .unwrap();
 
     let vals2 = &[Some(1.1), None, Some(3.3), None, None];
     let df2 = DataFrame::new(vec![
-        Series::new("date", &dts[3..]),
-        Series::new("val2", vals2),
+        Column::new("date".into(), &dts[3..]),
+        Column::new("val2".into(), vals2),
     ])
     .unwrap();
 
@@ -204,7 +229,7 @@ fn test_join_multiple_columns() {
         .str()
         .unwrap()
         + df_a.column("b").unwrap().str().unwrap();
-    s.rename("dummy");
+    s.rename("dummy".into());
 
     df_a.with_column(s).unwrap();
     let mut s = df_b
@@ -215,7 +240,7 @@ fn test_join_multiple_columns() {
         .str()
         .unwrap()
         + df_b.column("bar").unwrap().str().unwrap();
-    s.rename("dummy");
+    s.rename("dummy".into());
     df_b.with_column(s).unwrap();
 
     let joined = df_a.left_join(&df_b, ["dummy"], ["dummy"]).unwrap();
@@ -334,14 +359,14 @@ fn test_join_categorical() {
 fn test_empty_df_join() -> PolarsResult<()> {
     let empty: Vec<String> = vec![];
     let empty_df = DataFrame::new(vec![
-        Series::new("key", &empty),
-        Series::new("eval", &empty),
+        Column::new("key".into(), &empty),
+        Column::new("eval".into(), &empty),
     ])
     .unwrap();
 
     let df = DataFrame::new(vec![
-        Series::new("key", &["foo"]),
-        Series::new("aval", &[4]),
+        Column::new("key".into(), &["foo"]),
+        Column::new("aval".into(), &[4]),
     ])
     .unwrap();
 
@@ -357,8 +382,8 @@ fn test_empty_df_join() -> PolarsResult<()> {
 
     let empty: Vec<String> = vec![];
     let _empty_df = DataFrame::new(vec![
-        Series::new("key", &empty),
-        Series::new("eval", &empty),
+        Column::new("key".into(), &empty),
+        Column::new("eval".into(), &empty),
     ])
     .unwrap();
 
@@ -370,9 +395,9 @@ fn test_empty_df_join() -> PolarsResult<()> {
     // https://github.com/pola-rs/polars/issues/1824
     let empty: Vec<i32> = vec![];
     let empty_df = DataFrame::new(vec![
-        Series::new("key", &empty),
-        Series::new("1val", &empty),
-        Series::new("2val", &empty),
+        Column::new("key".into(), &empty),
+        Column::new("1val".into(), &empty),
+        Column::new("2val".into(), &empty),
     ])?;
 
     let out = df.left_join(&empty_df, ["key"], ["key"])?;
@@ -504,8 +529,8 @@ fn test_multi_joins_with_duplicates() -> PolarsResult<()> {
     let df_inner_join = df_left
         .join(
             &df_right,
-            &["col1", "join_col2"],
-            &["join_col1", "col2"],
+            ["col1", "join_col2"],
+            ["join_col1", "col2"],
             JoinType::Inner.into(),
         )
         .unwrap();
@@ -519,8 +544,8 @@ fn test_multi_joins_with_duplicates() -> PolarsResult<()> {
     let df_left_join = df_left
         .join(
             &df_right,
-            &["col1", "join_col2"],
-            &["join_col1", "col2"],
+            ["col1", "join_col2"],
+            ["join_col1", "col2"],
             JoinType::Left.into(),
         )
         .unwrap();
@@ -534,8 +559,8 @@ fn test_multi_joins_with_duplicates() -> PolarsResult<()> {
     let df_full_outer_join = df_left
         .join(
             &df_right,
-            &["col1", "join_col2"],
-            &["join_col1", "col2"],
+            ["col1", "join_col2"],
+            ["join_col1", "col2"],
             JoinArgs::new(JoinType::Full).with_coalesce(JoinCoalesce::CoalesceColumns),
         )
         .unwrap();
@@ -604,9 +629,9 @@ fn test_4_threads_bit_offset() -> PolarsResult<()> {
     let mut left_b = (0..n)
         .map(|i| if i % 2 == 0 { None } else { Some(0) })
         .collect::<Int64Chunked>();
-    left_a.rename("a");
-    left_b.rename("b");
-    let left_df = DataFrame::new(vec![left_a.into_series(), left_b.into_series()])?;
+    left_a.rename("a".into());
+    left_b.rename("b".into());
+    let left_df = DataFrame::new(vec![left_a.into_column(), left_b.into_column()])?;
 
     let i = 1;
     let len = 8;
@@ -615,10 +640,10 @@ fn test_4_threads_bit_offset() -> PolarsResult<()> {
     let mut right_b = range
         .map(|i| if i % 3 == 0 { None } else { Some(1) })
         .collect::<Int64Chunked>();
-    right_a.rename("a");
-    right_b.rename("b");
+    right_a.rename("a".into());
+    right_b.rename("b".into());
 
-    let right_df = DataFrame::new(vec![right_a.into_series(), right_b.into_series()])?;
+    let right_df = DataFrame::new(vec![right_a.into_column(), right_b.into_column()])?;
     let out = JoinBuilder::new(left_df.lazy())
         .with(right_df.lazy())
         .on([col("a"), col("b")])
