@@ -1,7 +1,7 @@
 use arrow::array::*;
-use arrow::chunk::Chunk;
 use arrow::datatypes::*;
 use arrow::io::avro::write;
+use arrow::record_batch::RecordBatchT;
 use avro_schema::file::Compression;
 use avro_schema::write_async::{write_block, write_metadata};
 use polars_error::PolarsResult;
@@ -10,7 +10,7 @@ use super::read::read_avro;
 use super::write::{data, schema, serialize_to_block};
 
 async fn write_avro<R: AsRef<dyn Array>>(
-    columns: &Chunk<R>,
+    columns: &RecordBatchT<R>,
     schema: &ArrowSchema,
     compression: Option<Compression>,
 ) -> PolarsResult<Vec<u8>> {
@@ -42,6 +42,9 @@ async fn roundtrip(compression: Option<Compression>) -> PolarsResult<()> {
     Ok(())
 }
 
+// Issue with clippy interacting with tokio. See:
+// https://github.com/rust-lang/rust-clippy/issues/13458
+#[allow(clippy::needless_return)]
 #[tokio::test]
 async fn no_compression() -> PolarsResult<()> {
     roundtrip(None).await
