@@ -203,7 +203,7 @@ fn test_apply_multiple_columns() -> PolarsResult<()> {
         .select([map_multiple(
             multiply,
             [col("A"), col("B")],
-            GetOutput::from_type(DataType::Float64),
+            GetOutput::from_type(DataType::Int32),
         )])
         .collect()?;
     let out = out.column("A")?;
@@ -219,7 +219,7 @@ fn test_apply_multiple_columns() -> PolarsResult<()> {
         .agg([apply_multiple(
             multiply,
             [col("A"), col("B")],
-            GetOutput::from_type(DataType::Float64),
+            GetOutput::from_type(DataType::Int32),
             true,
         )])
         .collect()?;
