@@ -3,12 +3,6 @@
 #![allow(non_local_definitions)]
 #![allow(clippy::too_many_arguments)] // Python functions can have many arguments due to default arguments
 
-#[cfg(feature = "build_info")]
-#[allow(dead_code)]
-mod build {
-    include!(concat!(env!("OUT_DIR"), "/built.rs"));
-}
-
 mod allocator;
 #[cfg(debug_assertions)]
 mod memory;
@@ -281,6 +275,10 @@ fn polars(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
     m.add_wrapped(wrap_pyfunction!(functions::set_random_seed))
         .unwrap();
 
+    // Functions - escape_regex
+    m.add_wrapped(wrap_pyfunction!(functions::escape_regex))
+        .unwrap();
+
     // Exceptions - Errors
     m.add(
         "PolarsError",
@@ -383,11 +381,12 @@ fn polars(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
     #[cfg(feature = "polars_cloud")]
     m.add_wrapped(wrap_pyfunction!(cloud::prepare_cloud_plan))
         .unwrap();
+    #[cfg(feature = "polars_cloud")]
+    m.add_wrapped(wrap_pyfunction!(cloud::_execute_ir_plan_with_gpu))
+        .unwrap();
 
     // Build info
     m.add("__version__", env!("CARGO_PKG_VERSION"))?;
-    #[cfg(feature = "build_info")]
-    add_build_info(py, m)?;
 
     // Plugins
     #[cfg(feature = "ffi_plugin")]
@@ -397,57 +396,7 @@ fn polars(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
     // Capsules
     m.add("_allocator", create_allocator_capsule(py)?)?;
 
-    Ok(())
-}
+    m.add("_debug", cfg!(debug_assertions))?;
 
-#[cfg(feature = "build_info")]
-fn add_build_info(py: Python, m: &Bound<PyModule>) -> PyResult<()> {
-    use pyo3::types::{PyDict, PyString};
-    let info = PyDict::new_bound(py);
-
-    let build = PyDict::new_bound(py);
-    build.set_item("rustc", build::RUSTC)?;
-    build.set_item("rustc-version", build::RUSTC_VERSION)?;
-    build.set_item("opt-level", build::OPT_LEVEL)?;
-    build.set_item("debug", build::DEBUG)?;
-    build.set_item("jobs", build::NUM_JOBS)?;
-    info.set_item("compiler", build)?;
-
-    info.set_item("time", build::BUILT_TIME_UTC)?;
-
-    let deps = PyDict::new_bound(py);
-    for (name, version) in build::DEPENDENCIES.iter() {
-        deps.set_item(name, version)?;
-    }
-    info.set_item("dependencies", deps)?;
-
-    let features = build::FEATURES
-        .iter()
-        .map(|feat| PyString::new_bound(py, feat))
-        .collect::<Vec<_>>();
-    info.set_item("features", features)?;
-
-    let host = PyDict::new_bound(py);
-    host.set_item("triple", build::HOST)?;
-    info.set_item("host", host)?;
-
-    let target = PyDict::new_bound(py);
-    target.set_item("arch", build::CFG_TARGET_ARCH)?;
-    target.set_item("os", build::CFG_OS)?;
-    target.set_item("family", build::CFG_FAMILY)?;
-    target.set_item("env", build::CFG_ENV)?;
-    target.set_item("triple", build::TARGET)?;
-    target.set_item("endianness", build::CFG_ENDIAN)?;
-    target.set_item("pointer-width", build::CFG_POINTER_WIDTH)?;
-    target.set_item("profile", build::PROFILE)?;
-    info.set_item("target", target)?;
-
-    let git = PyDict::new_bound(py);
-    git.set_item("version", build::GIT_VERSION)?;
-    git.set_item("dirty", build::GIT_DIRTY)?;
-    git.set_item("hash", build::GIT_COMMIT_HASH)?;
-    git.set_item("head", build::GIT_HEAD_REF)?;
-    info.set_item("git", git)?;
-    m.add("__build__", info)?;
     Ok(())
 }
