@@ -6,7 +6,7 @@ import random
 import string
 import sys
 import tracemalloc
-from typing import Any, Generator, List, cast
+from typing import TYPE_CHECKING, Any, cast
 
 import numpy as np
 import pytest
@@ -14,6 +14,9 @@ import pytest
 import polars as pl
 from polars.testing.parametric import load_profile
 
+if TYPE_CHECKING:
+    from collections.abc import Generator
+
 load_profile(
     profile=os.environ.get("POLARS_HYPOTHESIS_PROFILE", "fast"),  # type: ignore[arg-type]
 )
@@ -128,7 +131,7 @@ for T in ["T", " "]:
 
 @pytest.fixture(params=ISO8601_FORMATS_DATETIME)
 def iso8601_format_datetime(request: pytest.FixtureRequest) -> list[str]:
-    return cast(List[str], request.param)
+    return cast(list[str], request.param)
 
 
 ISO8601_TZ_AWARE_FORMATS_DATETIME = []
@@ -151,7 +154,7 @@ for T in ["T", " "]:
 
 @pytest.fixture(params=ISO8601_TZ_AWARE_FORMATS_DATETIME)
 def iso8601_tz_aware_format_datetime(request: pytest.FixtureRequest) -> list[str]:
-    return cast(List[str], request.param)
+    return cast(list[str], request.param)
 
 
 ISO8601_FORMATS_DATE = []
@@ -163,7 +166,7 @@ for date_sep in ("/", "-"):
 
 @pytest.fixture(params=ISO8601_FORMATS_DATE)
 def iso8601_format_date(request: pytest.FixtureRequest) -> list[str]:
-    return cast(List[str], request.param)
+    return cast(list[str], request.param)
 
 
 class MemoryUsage:
@@ -209,7 +212,7 @@ def memory_usage_without_pyarrow() -> Generator[MemoryUsage, Any, Any]:
 
     Memory usage from PyArrow is not tracked.
     """
-    if not pl.build_info()["compiler"]["debug"]:
+    if not pl.polars._debug:  # type: ignore[attr-defined]
         pytest.skip("Memory usage only available in debug/dev builds.")
 
     if os.getenv("POLARS_FORCE_ASYNC", "0") == "1":
