@@ -229,8 +229,10 @@ def test_rolling_crossing_dst(
 
 
 def test_rolling_by_invalid() -> None:
-    df = pl.DataFrame({"a": [1, 2, 3], "b": [4, 5, 6]}).sort("a")
-    msg = r"in `rolling_\*_by` operation, `by` argument of dtype `i64` is not supported"
+    df = pl.DataFrame(
+        {"a": [1, 2, 3], "b": [4, 5, 6]}, schema_overrides={"a": pl.Int16}
+    ).sort("a")
+    msg = "unsupported data type: i16 for `window_size`, expected UInt64, UInt32, Int64, Int32, Datetime, Date, Duration, or Time"
     with pytest.raises(InvalidOperationError, match=msg):
         df.select(pl.col("b").rolling_min_by("a", "2i"))
     df = pl.DataFrame({"a": [1, 2, 3], "b": [date(2020, 1, 1)] * 3}).sort("b")
@@ -818,6 +820,14 @@ def test_rolling_by_date() -> None:
     assert_frame_equal(result, expected)
 
 
+@pytest.mark.parametrize("dtype", [pl.Int64, pl.Int32, pl.UInt64, pl.UInt32])
+def test_rolling_by_integer(dtype: PolarsDataType) -> None:
+    df = pl.DataFrame({"val": [1, 2, 3]}).with_row_index()
+    result = df.with_columns(roll=pl.col("val").rolling_sum_by("index", "2i"))
+    expected = df.with_columns(roll=pl.Series([1, 3, 5]))
+    assert_frame_equal(result, expected)
+
+
 def test_rolling_nanoseconds_11003() -> None:
     df = pl.DataFrame(
         {
