@@ -2,7 +2,7 @@ from __future__ import annotations
 
 import math
 from contextlib import nullcontext
-from typing import TYPE_CHECKING, Any, ContextManager
+from typing import TYPE_CHECKING, Any
 
 import pytest
 
@@ -11,6 +11,8 @@ from polars.exceptions import ComputeError
 from polars.testing import assert_frame_equal
 
 if TYPE_CHECKING:
+    from contextlib import AbstractContextManager as ContextManager
+
     from polars._typing import PolarsDataType
 
 
@@ -158,6 +160,49 @@ def test_missing_equality_on_bools() -> None:
     ]
 
 
+def test_struct_equality_18870() -> None:
+    s = pl.Series([{"a": 1}, None])
+
+    # eq
+    result = s.eq(s).to_list()
+    expected = [True, None]
+    assert result == expected
+
+    # ne
+    result = s.ne(s).to_list()
+    expected = [False, None]
+    assert result == expected
+
+    # eq_missing
+    result = s.eq_missing(s).to_list()
+    expected = [True, True]
+    assert result == expected
+
+    # ne_missing
+    result = s.ne_missing(s).to_list()
+    expected = [False, False]
+    assert result == expected
+
+
+def test_struct_nested_equality() -> None:
+    df = pl.DataFrame(
+        {
+            "a": [{"foo": 0, "bar": "1"}, {"foo": None, "bar": "1"}, None],
+            "b": [{"foo": 0, "bar": "1"}] * 3,
+        }
+    )
+
+    # eq
+    ans = df.select(pl.col("a").eq(pl.col("b")))
+    expected = pl.DataFrame({"a": [True, False, None]})
+    assert_frame_equal(ans, expected)
+
+    # ne
+    ans = df.select(pl.col("a").ne(pl.col("b")))
+    expected = pl.DataFrame({"a": [False, True, None]})
+    assert_frame_equal(ans, expected)
+
+
 def isnan(x: Any) -> bool:
     return isinstance(x, float) and math.isnan(x)
 
@@ -392,3 +437,10 @@ def test_nested_binary_literal_super_type_12227() -> None:
 
     result = pl.select((pl.lit(0) + (pl.lit(0) == pl.lit(0)) * pl.lit(0.1)) + pl.lit(0))
     assert result.item() == 0.1
+
+
+def test_struct_broadcasting_comparison() -> None:
+    df = pl.DataFrame({"foo": [{"a": 1}, {"a": 2}, {"a": 1}]})
+    assert df.select(eq=pl.col.foo == pl.col.foo.last()).to_dict(as_series=False) == {
+        "eq": [True, False, True]
+    }
