@@ -466,7 +466,7 @@ def test_pivot_struct() -> None:
 
 def test_duplicate_column_names_which_should_raise_14305() -> None:
     df = pl.DataFrame({"a": [1, 3, 2], "c": ["a", "a", "a"], "d": [7, 8, 9]})
-    with pytest.raises(DuplicateError, match="has more than one occurrences"):
+    with pytest.raises(DuplicateError, match="has more than one occurrence"):
         df.pivot(index="a", on="c", values="d")
 
 
