@@ -96,9 +96,9 @@ def test_median_quantile_duration() -> None:
 def test_correlation_cast_supertype() -> None:
     df = pl.DataFrame({"a": [1, 8, 3], "b": [4.0, 5.0, 2.0]})
     df = df.with_columns(pl.col("b"))
-    assert df.select(pl.corr("a", "b")).to_dict(as_series=False) == {
-        "a": [0.5447047794019219]
-    }
+    assert_frame_equal(
+        df.select(pl.corr("a", "b")), pl.DataFrame({"a": [0.5447047794019219]})
+    )
 
 
 def test_cov_corr_f32_type() -> None:
