@@ -295,10 +295,13 @@ def test_join_misc_16255() -> None:
 )
 def test_non_equi_joins(constraint: str) -> None:
     # no support (yet) for non equi-joins in polars joins
-    with pytest.raises(
-        SQLInterfaceError,
-        match=r"only equi-join constraints are supported",
-    ), pl.SQLContext({"tbl": pl.DataFrame({"a": [1, 2, 3], "b": [4, 3, 2]})}) as ctx:
+    with (
+        pytest.raises(
+            SQLInterfaceError,
+            match=r"only equi-join constraints are supported",
+        ),
+        pl.SQLContext({"tbl": pl.DataFrame({"a": [1, 2, 3], "b": [4, 3, 2]})}) as ctx,
+    ):
         ctx.execute(
             f"""
             SELECT *
@@ -310,12 +313,19 @@ def test_non_equi_joins(constraint: str) -> None:
 
 def test_implicit_joins() -> None:
     # no support for this yet; ensure we catch it
-    with pytest.raises(
-        SQLInterfaceError,
-        match=r"not currently supported .* use explicit JOIN syntax instead",
-    ), pl.SQLContext(
-        {"tbl": pl.DataFrame({"a": [1, 2, 3], "b": [4, 3, 2], "c": ["x", "y", "z"]})}
-    ) as ctx:
+    with (
+        pytest.raises(
+            SQLInterfaceError,
+            match=r"not currently supported .* use explicit JOIN syntax instead",
+        ),
+        pl.SQLContext(
+            {
+                "tbl": pl.DataFrame(
+                    {"a": [1, 2, 3], "b": [4, 3, 2], "c": ["x", "y", "z"]}
+                )
+            }
+        ) as ctx,
+    ):
         ctx.execute(
             """
             SELECT t1.*
